@@ -1,7 +1,3 @@
-#include "src/function/scalar/string/pad.cpp"
-
-#include "src/function/scalar/string/prefix.cpp"
-
 #include "src/function/scalar/string/printf.cpp"
 
 #include "src/function/scalar/string/regexp.cpp"
@@ -24,6 +20,8 @@
 
 #include "src/function/scalar/string_functions.cpp"
 
+#include "src/function/scalar/system/system_functions.cpp"
+
 #include "src/function/scalar/trigonometrics_functions.cpp"
 
 #include "src/function/table/arrow.cpp"
@@ -34,14 +32,6 @@
 
 #include "src/function/table/glob.cpp"
 
-#include "src/function/table/information_schema/information_schema_columns.cpp"
-
-#include "src/function/table/information_schema/information_schema_schemata.cpp"
-
-#include "src/function/table/information_schema/information_schema_tables.cpp"
-
-#include "src/function/table/information_schema_functions.cpp"
-
 #include "src/function/table/pragma_detailed_profiling_output.cpp"
 
 #include "src/function/table/pragma_last_profiling_output.cpp"
@@ -52,24 +42,44 @@
 
 #include "src/function/table/repeat.cpp"
 
-#include "src/function/table/sqlite/pragma_collations.cpp"
-
-#include "src/function/table/sqlite/pragma_database_list.cpp"
-
-#include "src/function/table/sqlite/pragma_database_size.cpp"
-
-#include "src/function/table/sqlite/pragma_functions.cpp"
-
-#include "src/function/table/sqlite/pragma_table_info.cpp"
-
-#include "src/function/table/sqlite/sqlite_master.cpp"
-
-#include "src/function/table/sqlite_functions.cpp"
-
 #include "src/function/table/summary.cpp"
 
+#include "src/function/table/system/duckdb_columns.cpp"
+
+#include "src/function/table/system/duckdb_constraints.cpp"
+
+#include "src/function/table/system/duckdb_dependencies.cpp"
+
+#include "src/function/table/system/duckdb_indexes.cpp"
+
+#include "src/function/table/system/duckdb_schemas.cpp"
+
+#include "src/function/table/system/duckdb_sequences.cpp"
+
+#include "src/function/table/system/duckdb_tables.cpp"
+
+#include "src/function/table/system/duckdb_types.cpp"
+
+#include "src/function/table/system/duckdb_views.cpp"
+
+#include "src/function/table/system/pragma_collations.cpp"
+
+#include "src/function/table/system/pragma_database_list.cpp"
+
+#include "src/function/table/system/pragma_database_size.cpp"
+
+#include "src/function/table/system/pragma_functions.cpp"
+
+#include "src/function/table/system/pragma_storage_info.cpp"
+
+#include "src/function/table/system/pragma_table_info.cpp"
+
+#include "src/function/table/system_functions.cpp"
+
 #include "src/function/table/table_scan.cpp"
 
+#include "src/function/table/unnest.cpp"
+
 #include "src/function/table/version/pragma_version.cpp"
 
 #include "src/function/udf_function.cpp"
@@ -78,6 +88,8 @@
 
 #include "src/main/client_context.cpp"
 
+#include "src/main/config.cpp"
+
 #include "src/main/connection.cpp"
 
 #include "src/main/database.cpp"
@@ -192,9 +204,3 @@
 
 #include "src/optimizer/pushdown/pushdown_inner_join.cpp"
 
-#include "src/optimizer/pushdown/pushdown_left_join.cpp"
-
-#include "src/optimizer/pushdown/pushdown_mark_join.cpp"
-
-#include "src/optimizer/pushdown/pushdown_projection.cpp"
-
