@@ -12,12 +12,7 @@ IndexCatalogEntry::~IndexCatalogEntry() {
 	if (!info || !index) {
 		return;
 	}
-	for (idx_t i = 0; i < info->indexes.size(); i++) {
-		if (info->indexes[i].get() == index) {
-			info->indexes.erase(info->indexes.begin() + i);
-			break;
-		}
-	}
+	info->indexes.RemoveIndex(index);
 }
 
 string IndexCatalogEntry::ToSQL() {
