@@ -7,7 +7,7 @@ struct DefaultSchema {
 	const char *name;
 };
 
-static DefaultSchema internal_schemas[] = {{"information_schema"}, {nullptr}};
+static DefaultSchema internal_schemas[] = {{"information_schema"}, {"pg_catalog"}, {nullptr}};
 
 static bool GetDefaultSchema(const string &schema) {
 	for (idx_t index = 0; internal_schemas[index].name != nullptr; index++) {
@@ -28,4 +28,12 @@ unique_ptr<CatalogEntry> DefaultSchemaGenerator::CreateDefaultEntry(ClientContex
 	return nullptr;
 }
 
+vector<string> DefaultSchemaGenerator::GetDefaultEntries() {
+	vector<string> result;
+	for (idx_t index = 0; internal_schemas[index].name != nullptr; index++) {
+		result.emplace_back(internal_schemas[index].name);
+	}
+	return result;
+}
+
 } // namespace duckdb
