@@ -1,15 +1,16 @@
 #include "duckdb/common/file_system.hpp"
 
+#include "duckdb/common/checksum.hpp"
 #include "duckdb/common/exception.hpp"
+#include "duckdb/common/gzip_file_system.hpp"
 #include "duckdb/common/helper.hpp"
 #include "duckdb/common/string_util.hpp"
-#include "duckdb/common/checksum.hpp"
+#include "duckdb/function/scalar/string_functions.hpp"
 #include "duckdb/main/client_context.hpp"
 #include "duckdb/main/database.hpp"
-#include "duckdb/function/scalar/string_functions.hpp"
 
-#include <cstdio>
 #include <cstdint>
+#include <cstdio>
 
 #ifndef _WIN32
 #include <dirent.h>
@@ -83,7 +84,12 @@ public:
 	int fd;
 };
 
-unique_ptr<FileHandle> FileSystem::OpenFile(const char *path, uint8_t flags, FileLockType lock_type) {
+unique_ptr<FileHandle> FileSystem::OpenFile(const string &path, uint8_t flags, FileLockType lock_type,
+                                            FileCompressionType compression) {
+	if (compression != FileCompressionType::UNCOMPRESSED) {
+		throw NotImplementedException("Unsupported compression type for default file system");
+	}
+
 	AssertValidFileFlags(flags);
 
 	int open_flags = 0;
@@ -114,7 +120,7 @@ unique_ptr<FileHandle> FileSystem::OpenFile(const char *path, uint8_t flags, Fil
 		open_flags |= O_DIRECT | O_SYNC;
 #endif
 	}
-	int fd = open(path, open_flags, 0666);
+	int fd = open(path.c_str(), open_flags, 0666);
 	if (fd == -1) {
 		throw IOException("Cannot open file \"%s\": %s", path, strerror(errno));
 	}
@@ -152,6 +158,27 @@ void FileSystem::SetFilePointer(FileHandle &handle, idx_t location) {
 	}
 }
 
+idx_t FileSystem::GetFilePointer(FileHandle &handle) {
+	int fd = ((UnixFileHandle &)handle).fd;
+	off_t position = lseek(fd, 0, SEEK_CUR);
+	if (position == (off_t)-1) {
+		throw IOException("Could not get file position file \"%s\": %s", handle.path, strerror(errno));
+	}
+	return position;
+}
+
+void FileSystem::Read(FileHandle &handle, void *buffer, int64_t nr_bytes, idx_t location) {
+	int fd = ((UnixFileHandle &)handle).fd;
+	int64_t bytes_read = pread(fd, buffer, nr_bytes, location);
+	if (bytes_read == -1) {
+		throw IOException("Could not read from file \"%s\": %s", handle.path, strerror(errno));
+	}
+	if (bytes_read != nr_bytes) {
+		throw IOException("Could not read all bytes from file \"%s\": wanted=%lld read=%lld", handle.path, nr_bytes,
+		                  bytes_read);
+	}
+}
+
 int64_t FileSystem::Read(FileHandle &handle, void *buffer, int64_t nr_bytes) {
 	int fd = ((UnixFileHandle &)handle).fd;
 	int64_t bytes_read = read(fd, buffer, nr_bytes);
@@ -161,6 +188,18 @@ int64_t FileSystem::Read(FileHandle &handle, void *buffer, int64_t nr_bytes) {
 	return bytes_read;
 }
 
+void FileSystem::Write(FileHandle &handle, void *buffer, int64_t nr_bytes, idx_t location) {
+	int fd = ((UnixFileHandle &)handle).fd;
+	int64_t bytes_written = pwrite(fd, buffer, nr_bytes, location);
+	if (bytes_written == -1) {
+		throw IOException("Could not write file \"%s\": %s", handle.path, strerror(errno));
+	}
+	if (bytes_written != nr_bytes) {
+		throw IOException("Could not write all bytes to file \"%s\": wanted=%lld wrote=%lld", handle.path, nr_bytes,
+		                  bytes_written);
+	}
+}
+
 int64_t FileSystem::Write(FileHandle &handle, void *buffer, int64_t nr_bytes) {
 	int fd = ((UnixFileHandle &)handle).fd;
 	int64_t bytes_written = write(fd, buffer, nr_bytes);
@@ -385,7 +424,8 @@ std::string GetLastErrorAsString() {
 
 struct WindowsFileHandle : public FileHandle {
 public:
-	WindowsFileHandle(FileSystem &file_system, string path, HANDLE fd) : FileHandle(file_system, path), fd(fd) {
+	WindowsFileHandle(FileSystem &file_system, string path, HANDLE fd)
+	    : FileHandle(file_system, path), position(0), fd(fd) {
 	}
 	virtual ~WindowsFileHandle() {
 		Close();
@@ -397,16 +437,21 @@ protected:
 	};
 
 public:
+	idx_t position;
 	HANDLE fd;
 };
 
-unique_ptr<FileHandle> FileSystem::OpenFile(const char *path, uint8_t flags, FileLockType lock_type) {
+unique_ptr<FileHandle> FileSystem::OpenFile(const string &path, uint8_t flags, FileLockType lock_type,
+                                            FileCompressionType compression) {
+	if (compression != FileCompressionType::UNCOMPRESSED) {
+		throw NotImplementedException("Unsupported compression type for default file system");
+	}
 	AssertValidFileFlags(flags);
 
 	DWORD desired_access;
 	DWORD share_mode;
 	DWORD creation_disposition = OPEN_EXISTING;
-	DWORD flags_and_attributes = FILE_ATTRIBUTE_NORMAL;
+	DWORD flags_and_attributes = FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED;
 	if (flags & FileFlags::FILE_FLAGS_READ) {
 		desired_access = GENERIC_READ;
 		share_mode = FILE_SHARE_READ;
@@ -428,49 +473,108 @@ unique_ptr<FileHandle> FileSystem::OpenFile(const char *path, uint8_t flags, Fil
 		flags_and_attributes |= FILE_FLAG_NO_BUFFERING;
 	}
 	HANDLE hFile =
-	    CreateFileA(path, desired_access, share_mode, NULL, creation_disposition, flags_and_attributes, NULL);
+	    CreateFileA(path.c_str(), desired_access, share_mode, NULL, creation_disposition, flags_and_attributes, NULL);
 	if (hFile == INVALID_HANDLE_VALUE) {
 		auto error = GetLastErrorAsString();
-		throw IOException("Cannot open file \"%s\": %s", path, error);
+		throw IOException("Cannot open file \"%s\": %s", path.c_str(), error);
 	}
-	auto handle = make_unique<WindowsFileHandle>(*this, path, hFile);
+	auto handle = make_unique<WindowsFileHandle>(*this, path.c_str(), hFile);
 	if (flags & FileFlags::FILE_FLAGS_APPEND) {
-		SetFilePointer(*handle, GetFileSize(*handle));
+		auto file_size = GetFileSize(*handle);
+		SetFilePointer(*handle, file_size);
 	}
 	return move(handle);
 }
 
 void FileSystem::SetFilePointer(FileHandle &handle, idx_t location) {
+	((WindowsFileHandle &)handle).position = location;
+}
+
+idx_t FileSystem::GetFilePointer(FileHandle &handle) {
+	return ((WindowsFileHandle &)handle).position;
+}
+
+void FileSystem::Read(FileHandle &handle, void *buffer, int64_t nr_bytes, idx_t location) {
 	HANDLE hFile = ((WindowsFileHandle &)handle).fd;
-	LARGE_INTEGER loc;
-	loc.QuadPart = location;
-	auto rc = SetFilePointerEx(hFile, loc, NULL, FILE_BEGIN);
+	DWORD bytes_read;
+	OVERLAPPED ov = {};
+	ov.Internal = 0;
+	ov.InternalHigh = 0;
+	ov.Offset = location & 0xFFFFFFFF;
+	ov.OffsetHigh = location >> 32;
+	ov.hEvent = 0;
+	ReadFile(hFile, buffer, (DWORD)nr_bytes, NULL, &ov);
+	auto rc = GetOverlappedResult(hFile, &ov, &bytes_read, true);
 	if (rc == 0) {
 		auto error = GetLastErrorAsString();
-		throw IOException("Could not seek to location %lld for file \"%s\": %s", location, handle.path, error);
+		throw IOException("Could not read file \"%s\": %s", handle.path, error);
+	}
+	if (bytes_read != nr_bytes) {
+		throw IOException("Could not read all bytes from file \"%s\": wanted=%lld read=%lld", handle.path, nr_bytes,
+		                  bytes_read);
 	}
 }
 
 int64_t FileSystem::Read(FileHandle &handle, void *buffer, int64_t nr_bytes) {
 	HANDLE hFile = ((WindowsFileHandle &)handle).fd;
 	DWORD bytes_read;
-	auto rc = ReadFile(hFile, buffer, (DWORD)nr_bytes, &bytes_read, NULL);
+	auto &pos = ((WindowsFileHandle &)handle).position;
+	OVERLAPPED ov = {};
+	ov.Internal = 0;
+	ov.InternalHigh = 0;
+	ov.Offset = pos & 0xFFFFFFFF;
+	ov.OffsetHigh = pos >> 32;
+	ov.hEvent = 0;
+	auto n = std::min<idx_t>(std::max<idx_t>(GetFileSize(handle), pos) - pos, nr_bytes);
+	ReadFile(hFile, buffer, (DWORD)n, NULL, &ov);
+	auto rc = GetOverlappedResult(hFile, &ov, &bytes_read, true);
+	if (rc == 0) {
+		auto error = GetLastErrorAsString();
+		throw IOException("Could not read file \"%s\": %s", handle.path, error);
+	}
+	pos += bytes_read;
+	return bytes_read;
+}
+
+void FileSystem::Write(FileHandle &handle, void *buffer, int64_t nr_bytes, idx_t location) {
+	HANDLE hFile = ((WindowsFileHandle &)handle).fd;
+	DWORD bytes_written;
+	OVERLAPPED ov = {};
+	ov.Internal = 0;
+	ov.InternalHigh = 0;
+	ov.Offset = location & 0xFFFFFFFF;
+	ov.OffsetHigh = location >> 32;
+	ov.hEvent = 0;
+	WriteFile(hFile, buffer, (DWORD)nr_bytes, NULL, &ov);
+	auto rc = GetOverlappedResult(hFile, &ov, &bytes_written, true);
 	if (rc == 0) {
 		auto error = GetLastErrorAsString();
 		throw IOException("Could not write file \"%s\": %s", handle.path, error);
 	}
-	return bytes_read;
+	if (bytes_written != nr_bytes) {
+		throw IOException("Could not write all bytes from file \"%s\": wanted=%lld wrote=%lld", handle.path, nr_bytes,
+		                  bytes_written);
+	}
 }
 
 int64_t FileSystem::Write(FileHandle &handle, void *buffer, int64_t nr_bytes) {
 	HANDLE hFile = ((WindowsFileHandle &)handle).fd;
-	DWORD bytes_read;
-	auto rc = WriteFile(hFile, buffer, (DWORD)nr_bytes, &bytes_read, NULL);
+	DWORD bytes_written;
+	auto &pos = ((WindowsFileHandle &)handle).position;
+	OVERLAPPED ov = {};
+	ov.Internal = 0;
+	ov.InternalHigh = 0;
+	ov.Offset = pos & 0xFFFFFFFF;
+	ov.OffsetHigh = pos >> 32;
+	ov.hEvent = 0;
+	WriteFile(hFile, buffer, (DWORD)nr_bytes, NULL, &ov);
+	auto rc = GetOverlappedResult(hFile, &ov, &bytes_written, true);
 	if (rc == 0) {
 		auto error = GetLastErrorAsString();
 		throw IOException("Could not write file \"%s\": %s", handle.path, error);
 	}
-	return bytes_read;
+	pos += bytes_written;
+	return bytes_written;
 }
 
 int64_t FileSystem::GetFileSize(FileHandle &handle) {
@@ -671,24 +775,30 @@ string FileSystem::GetHomeDirectory() {
 	return homedir;
 }
 
-void FileSystem::Read(FileHandle &handle, void *buffer, int64_t nr_bytes, idx_t location) {
-	// seek to the location
-	SetFilePointer(handle, location);
-	// now read from the location
-	int64_t bytes_read = Read(handle, buffer, nr_bytes);
-	if (bytes_read != nr_bytes) {
-		throw IOException("Could not read sufficient bytes from file \"%s\"", handle.path);
-	}
+bool FileSystem::CanSeek() {
+	return true;
 }
 
-void FileSystem::Write(FileHandle &handle, void *buffer, int64_t nr_bytes, idx_t location) {
-	// seek to the location
-	SetFilePointer(handle, location);
-	// now write to the location
-	int64_t bytes_written = Write(handle, buffer, nr_bytes);
-	if (bytes_written != nr_bytes) {
-		throw IOException("Could not write sufficient bytes from file \"%s\"", handle.path);
+bool FileSystem::OnDiskFile(FileHandle &handle) {
+	return true;
+}
+
+void FileSystem::Seek(FileHandle &handle, idx_t location) {
+	if (!CanSeek()) {
+		throw IOException("Cannot seek in files of this type");
 	}
+	SetFilePointer(handle, location);
+}
+
+void FileSystem::Reset(FileHandle &handle) {
+	Seek(handle, 0);
+}
+
+idx_t FileSystem::SeekPosition(FileHandle &handle) {
+	if (!CanSeek()) {
+		throw IOException("Cannot seek in files of this type");
+	}
+	return GetFilePointer(handle);
 }
 
 string FileSystem::JoinPath(const string &a, const string &b) {
@@ -719,6 +829,14 @@ string FileSystem::ExtractBaseName(const string &path) {
 	return vec[0];
 }
 
+int64_t FileHandle::Read(void *buffer, idx_t nr_bytes) {
+	return file_system.Read(*this, buffer, nr_bytes);
+}
+
+int64_t FileHandle::Write(void *buffer, idx_t nr_bytes) {
+	return file_system.Write(*this, buffer, nr_bytes);
+}
+
 void FileHandle::Read(void *buffer, idx_t nr_bytes, idx_t location) {
 	file_system.Read(*this, buffer, nr_bytes, location);
 }
@@ -727,6 +845,44 @@ void FileHandle::Write(void *buffer, idx_t nr_bytes, idx_t location) {
 	file_system.Write(*this, buffer, nr_bytes, location);
 }
 
+void FileHandle::Seek(idx_t location) {
+	file_system.Seek(*this, location);
+}
+
+void FileHandle::Reset() {
+	file_system.Reset(*this);
+}
+
+idx_t FileHandle::SeekPosition() {
+	return file_system.SeekPosition(*this);
+}
+
+bool FileHandle::CanSeek() {
+	return file_system.CanSeek();
+}
+
+string FileHandle::ReadLine() {
+	string result;
+	char buffer[1];
+	while (true) {
+		idx_t tuples_read = Read(buffer, 1);
+		if (tuples_read == 0 || buffer[0] == '\n') {
+			return result;
+		}
+		if (buffer[0] != '\r') {
+			result += buffer[0];
+		}
+	}
+}
+
+bool FileHandle::OnDiskFile() {
+	return file_system.OnDiskFile(*this);
+}
+
+idx_t FileHandle::GetFileSize() {
+	return file_system.GetFileSize(*this);
+}
+
 void FileHandle::Sync() {
 	file_system.FileSync(*this);
 }
@@ -853,4 +1009,29 @@ vector<string> FileSystem::Glob(const string &path) {
 	return vector<string>();
 }
 
+unique_ptr<FileHandle> VirtualFileSystem::OpenFile(const string &path, uint8_t flags, FileLockType lock,
+                                                   FileCompressionType compression) {
+	if (compression == FileCompressionType::AUTO_DETECT) {
+		// auto detect compression settings based on file name
+		auto lower_path = StringUtil::Lower(path);
+		if (StringUtil::EndsWith(lower_path, ".gz")) {
+			compression = FileCompressionType::GZIP;
+		} else {
+			compression = FileCompressionType::UNCOMPRESSED;
+		}
+	}
+	// open the base file handle
+	auto file_handle = FindFileSystem(path)->OpenFile(path, flags, lock, FileCompressionType::UNCOMPRESSED);
+	if (compression != FileCompressionType::UNCOMPRESSED) {
+		switch (compression) {
+		case FileCompressionType::GZIP:
+			file_handle = GZipFileSystem::OpenCompressedFile(move(file_handle));
+			break;
+		default:
+			throw NotImplementedException("Unimplemented compression type");
+		}
+	}
+	return file_handle;
+}
+
 } // namespace duckdb
