@@ -5,6 +5,8 @@
 
 namespace duckdb {
 
+using ValidityBytes = TemplatedValidityMask<uint8_t>;
+
 RowDataCollection::RowDataCollection(BufferManager &buffer_manager, idx_t block_capacity, idx_t entry_size)
     : buffer_manager(buffer_manager), count(0), block_capacity(block_capacity), entry_size(entry_size),
       is_little_endian(IsLittleEndian()) {
@@ -184,7 +186,7 @@ void RowDataCollection::ComputeStructEntrySizes(Vector &v, idx_t entry_sizes[],
 		num_children = children.size();
 		for (auto &struct_child : children) {
 			Vector struct_vector;
-			struct_vector.Slice(*struct_child.second, dict_sel, vcount);
+			struct_vector.Slice(*struct_child, dict_sel, vcount);
 			struct_vectors.push_back(move(struct_vector));
 		}
 	} else {
@@ -192,7 +194,7 @@ void RowDataCollection::ComputeStructEntrySizes(Vector &v, idx_t entry_sizes[],
 		num_children = children.size();
 		for (auto &struct_child : children) {
 			Vector struct_vector;
-			struct_vector.Reference(*struct_child.second);
+			struct_vector.Reference(*struct_child);
 			struct_vectors.push_back(move(struct_vector));
 		}
 	}
@@ -217,6 +219,8 @@ static list_entry_t *GetListData(Vector &v) {
 }
 
 void RowDataCollection::ComputeListEntrySizes(Vector &v, idx_t entry_sizes[], idx_t vcount, idx_t offset) {
+	ListVector::Initialize(v);
+
 	VectorData vdata;
 	v.Orrify(vcount, vdata);
 
@@ -233,7 +237,7 @@ void RowDataCollection::ComputeListEntrySizes(Vector &v, idx_t entry_sizes[], id
 			entry_sizes[i] += (list_entry.length + 7) / 8;
 
 			// serialize size of each entry (if non-constant size)
-			if (!TypeIsConstantSize(v.GetType().child_types()[0].second.InternalType())) {
+			if (!TypeIsConstantSize(ListType::GetChildType(v.GetType()).InternalType())) {
 				entry_sizes[i] += list_entry.length * sizeof(list_entry.length);
 			}
 
@@ -444,7 +448,7 @@ void RowDataCollection::SerializeStructVector(Vector &v, idx_t vcount, const Sel
 		num_children = children.size();
 		for (auto &struct_child : children) {
 			Vector struct_vector;
-			struct_vector.Slice(*struct_child.second, dict_sel, vcount);
+			struct_vector.Slice(*struct_child, dict_sel, vcount);
 			struct_vectors.push_back(move(struct_vector));
 		}
 	} else {
@@ -452,7 +456,7 @@ void RowDataCollection::SerializeStructVector(Vector &v, idx_t vcount, const Sel
 		num_children = children.size();
 		for (auto &struct_child : children) {
 			Vector struct_vector;
-			struct_vector.Reference(*struct_child.second);
+			struct_vector.Reference(*struct_child);
 			struct_vectors.push_back(move(struct_vector));
 		}
 	}
@@ -486,20 +490,22 @@ void RowDataCollection::SerializeStructVector(Vector &v, idx_t vcount, const Sel
 }
 
 void RowDataCollection::SerializeListVector(Vector &v, idx_t vcount, const SelectionVector &sel, idx_t ser_count,
-                                            idx_t col_idx, data_ptr_t key_locations[],
+                                            idx_t col_no, data_ptr_t key_locations[],
                                             data_ptr_t validitymask_locations[], idx_t offset) {
 	VectorData vdata;
 	v.Orrify(vcount, vdata);
 
-	auto byte_offset = col_idx / 8;
-	const auto bit = ~(1UL << (col_idx % 8));
+	idx_t entry_idx;
+	idx_t idx_in_entry;
+	ValidityBytes::GetEntryIndex(col_no, entry_idx, idx_in_entry);
 
 	auto list_data = GetListData(v);
+	ListVector::Initialize(v);
 	auto &child_vector = ListVector::GetEntry(v);
 
 	VectorData list_vdata;
 	child_vector.Orrify(ListVector::GetListSize(v), list_vdata);
-	auto child_type = v.GetType().child_types()[0].second.InternalType();
+	auto child_type = ListType::GetChildType(v.GetType()).InternalType();
 
 	idx_t list_entry_sizes[STANDARD_VECTOR_SIZE];
 	data_ptr_t list_entry_locations[STANDARD_VECTOR_SIZE];
@@ -509,8 +515,9 @@ void RowDataCollection::SerializeListVector(Vector &v, idx_t vcount, const Selec
 		auto source_idx = vdata.sel->get_index(idx) + offset;
 		if (!vdata.validity.RowIsValid(source_idx)) {
 			if (validitymask_locations) {
-				// set the validitymask
-				*(validitymask_locations[i] + byte_offset) &= bit;
+				// set the row validitymask for this column to invalid
+				ValidityBytes row_mask(validitymask_locations[i]);
+				row_mask.SetInvalidUnsafe(entry_idx, idx_in_entry);
 			}
 			continue;
 		}
@@ -572,8 +579,8 @@ void RowDataCollection::SerializeListVector(Vector &v, idx_t vcount, const Selec
 			}
 
 			// now serialize to the locations
-			SerializeVector(child_vector, ListVector::GetListSize(v), sel, next, 0, list_entry_locations, nullptr,
-			                entry_offset);
+			SerializeVector(child_vector, ListVector::GetListSize(v), FlatVector::INCREMENTAL_SELECTION_VECTOR, next, 0,
+			                list_entry_locations, nullptr, entry_offset);
 
 			// update for next iteration
 			entry_remaining -= next;
@@ -693,70 +700,61 @@ void RowDataCollection::Build(idx_t added_count, data_ptr_t key_locations[], idx
 	}
 }
 
-template <class T>
-static void TemplatedDeserializeIntoVector(Vector &v, idx_t count, idx_t col_idx, data_ptr_t *key_locations) {
-	auto target = FlatVector::GetData<T>(v);
-	// fixed-size inner loop to allow unrolling
-	idx_t i;
-	for (i = 0; i + 7 < count; i += 8) {
-		for (idx_t j = 0; j < 8; j++) {
-			target[i + j] = Load<T>(key_locations[i + j]);
-			key_locations[i + j] += sizeof(T);
-		}
+void RowDataCollection::Merge(RowDataCollection &other) {
+	RowDataCollection temp(buffer_manager, Storage::BLOCK_ALLOC_SIZE, 1);
+	{
+		//	One lock at a time to avoid deadlocks
+		lock_guard<mutex> read_lock(other.rc_lock);
+		temp.count = other.count;
+		temp.block_capacity = other.block_capacity;
+		temp.entry_size = other.entry_size;
+		temp.blocks = move(other.blocks);
+		other.count = 0;
 	}
-	// finishing up
-	for (; i < count; i++) {
-		target[i] = Load<T>(key_locations[i]);
+
+	lock_guard<mutex> write_lock(rc_lock);
+	count += temp.count;
+	block_capacity = MaxValue(block_capacity, temp.block_capacity);
+	entry_size = MaxValue(entry_size, temp.entry_size);
+	for (auto &block : temp.blocks) {
+		blocks.emplace_back(move(block));
+	}
+}
+
+template <class T>
+static void TemplatedDeserializeIntoVector(Vector &v, const idx_t count, const SelectionVector &sel,
+                                           data_ptr_t *key_locations) {
+	auto target = FlatVector::GetData<T>(v);
+
+	for (idx_t i = 0; i < count; ++i) {
+		const auto col_idx = sel.get_index(i);
+		target[col_idx] = Load<T>(key_locations[i]);
 		key_locations[i] += sizeof(T);
 	}
 }
 
-static ValidityMask &GetValidity(Vector &v) {
-	switch (v.GetVectorType()) {
-	case VectorType::DICTIONARY_VECTOR:
-		return GetValidity(DictionaryVector::Child(v));
-	case VectorType::FLAT_VECTOR:
-		return FlatVector::Validity(v);
-	case VectorType::CONSTANT_VECTOR:
-		return ConstantVector::Validity(v);
-	default:
-		throw NotImplementedException("FIXME: cannot deserialize vector with this vectortype");
-	}
-}
-
-void RowDataCollection::DeserializeIntoStringVector(Vector &v, const idx_t &vcount, const idx_t &col_idx,
-                                                    data_ptr_t *key_locations, data_ptr_t *validitymask_locations) {
+static void DeserializeIntoStringVector(Vector &v, const idx_t vcount, const SelectionVector &sel,
+                                        data_ptr_t *key_locations) {
 	const auto &validity = FlatVector::Validity(v);
 	const idx_t string_prefix_len = string_t::PREFIX_LENGTH;
 	auto target = FlatVector::GetData<string_t>(v);
-	// fixed size inner loop to allow unrolling
-	idx_t i = 0;
-	if (validity.AllValid()) {
-		for (; i + 7 < vcount; i += 8) {
-			for (idx_t j = 0; j < 8; j++) {
-				auto len = Load<uint32_t>(key_locations[i + j]);
-				key_locations[i + j] += string_prefix_len;
-				target[i + j] = StringVector::AddStringOrBlob(v, string_t((const char *)key_locations[i + j], len));
-				key_locations[i + j] += len;
-			}
-		}
-	}
-	// finishing up
-	for (; i < vcount; i++) {
-		if (!validity.RowIsValid(i)) {
+
+	for (idx_t i = 0; i < vcount; i++) {
+		const auto col_idx = sel.get_index(i);
+		if (!validity.RowIsValid(col_idx)) {
 			continue;
 		}
 		auto len = Load<uint32_t>(key_locations[i]);
 		key_locations[i] += string_prefix_len;
-		target[i] = StringVector::AddStringOrBlob(v, string_t((const char *)key_locations[i], len));
+		target[col_idx] = StringVector::AddStringOrBlob(v, string_t((const char *)key_locations[i], len));
 		key_locations[i] += len;
 	}
 }
 
-void RowDataCollection::DeserializeIntoStructVector(Vector &v, const idx_t &vcount, const idx_t &col_idx,
-                                                    data_ptr_t *key_locations, data_ptr_t *validitymask_locations) {
+static void DeserializeIntoStructVector(Vector &v, const idx_t vcount, const SelectionVector &sel,
+                                        data_ptr_t *key_locations) {
 	// struct must have a validitymask for its fields
-	auto &child_types = v.GetType().child_types();
+	auto &child_types = StructType::GetChildTypes(v.GetType());
 	const idx_t struct_validitymask_size = (child_types.size() + 7) / 8;
 	data_ptr_t struct_validitymask_locations[STANDARD_VECTOR_SIZE];
 	for (idx_t i = 0; i < vcount; i++) {
@@ -766,33 +764,34 @@ void RowDataCollection::DeserializeIntoStructVector(Vector &v, const idx_t &vcou
 	}
 
 	// now deserialize into the struct vectors
+	auto &children = StructVector::GetEntries(v);
 	for (idx_t i = 0; i < child_types.size(); i++) {
-		auto new_child = make_unique<Vector>(child_types[i].second);
-		DeserializeIntoVector(*new_child, vcount, i, key_locations, struct_validitymask_locations);
-		StructVector::AddEntry(v, child_types[i].first, move(new_child));
+		RowDataCollection::DeserializeIntoVector(*children[i], vcount, sel, i, key_locations,
+		                                         struct_validitymask_locations);
 	}
 }
 
-void RowDataCollection::DeserializeIntoListVector(Vector &v, const idx_t &vcount, const idx_t &col_idx,
-                                                  data_ptr_t *key_locations, data_ptr_t *validitymask_locations) {
+static void DeserializeIntoListVector(Vector &v, const idx_t vcount, const SelectionVector &sel,
+                                      data_ptr_t *key_locations) {
 	const auto &validity = FlatVector::Validity(v);
 
-	auto child_type = v.GetType().child_types()[0].second;
+	auto child_type = ListType::GetChildType(v.GetType());
 	auto list_data = GetListData(v);
 	data_ptr_t list_entry_locations[STANDARD_VECTOR_SIZE];
 
 	ListVector::Initialize(v);
 	uint64_t entry_offset = ListVector::GetListSize(v);
 	for (idx_t i = 0; i < vcount; i++) {
-		if (!validity.RowIsValid(i)) {
+		const auto col_idx = sel.get_index(i);
+		if (!validity.RowIsValid(col_idx)) {
 			continue;
 		}
 		// read list length
 		auto entry_remaining = Load<uint64_t>(key_locations[i]);
 		key_locations[i] += sizeof(uint64_t);
 		// set list entry attributes
-		list_data[i].length = entry_remaining;
-		list_data[i].offset = entry_offset;
+		list_data[col_idx].length = entry_remaining;
+		list_data[col_idx].offset = entry_offset;
 		// skip over the validity mask
 		data_ptr_t validitymask_location = key_locations[i];
 		idx_t offset_in_byte = 0;
@@ -815,7 +814,8 @@ void RowDataCollection::DeserializeIntoListVector(Vector &v, const idx_t &vcount
 			auto &list_vec_to_append = ListVector::GetEntry(append_vector);
 
 			// set validity
-			auto &append_validity = GetValidity(list_vec_to_append);
+			//! Since we are constructing the vector, this will always be a flat vector.
+			auto &append_validity = FlatVector::Validity(list_vec_to_append);
 			for (idx_t entry_idx = 0; entry_idx < next; entry_idx++) {
 				append_validity.Set(entry_idx, *(validitymask_location) & (1 << offset_in_byte));
 				if (++offset_in_byte == 8) {
@@ -842,7 +842,8 @@ void RowDataCollection::DeserializeIntoListVector(Vector &v, const idx_t &vcount
 			}
 
 			// now deserialize and add to listvector
-			DeserializeIntoVector(list_vec_to_append, next, 0, list_entry_locations, nullptr);
+			RowDataCollection::DeserializeIntoVector(list_vec_to_append, next, FlatVector::INCREMENTAL_SELECTION_VECTOR,
+			                                         0, list_entry_locations, nullptr);
 			ListVector::Append(v, list_vec_to_append, next);
 
 			// update for next iteration
@@ -852,27 +853,23 @@ void RowDataCollection::DeserializeIntoListVector(Vector &v, const idx_t &vcount
 	}
 }
 
-void RowDataCollection::DeserializeIntoVector(Vector &v, const idx_t &vcount, const idx_t &col_idx,
-                                              data_ptr_t key_locations[], data_ptr_t validitymask_locations[]) {
+void RowDataCollection::DeserializeIntoVector(Vector &v, const idx_t &vcount, const SelectionVector &sel,
+                                              const idx_t &col_no, data_ptr_t key_locations[],
+                                              data_ptr_t validitymask_locations[]) {
+	v.SetVectorType(VectorType::FLAT_VECTOR);
+
 	auto &validity = FlatVector::Validity(v);
 	if (validitymask_locations) {
-		// validity mask is not yet set: deserialize it
-		const auto byte_offset = col_idx / 8;
-		const auto bit = 1 << (col_idx % 8);
+		// Precompute mask indexes
+		idx_t entry_idx;
+		idx_t idx_in_entry;
+		ValidityBytes::GetEntryIndex(col_no, entry_idx, idx_in_entry);
 
-		// fixed-size inner loop to allow unrolling
-		idx_t i;
-		for (i = 0; i + 7 < vcount; i += 8) {
-			for (idx_t j = 0; j < 8; j++) {
-				bool valid = *(validitymask_locations[i + j] + byte_offset) & bit;
-				validity.Set(i + j, valid);
-			}
-		}
-
-		// finishing up
-		for (i = 0; i < vcount; i++) {
-			bool valid = *(validitymask_locations[i] + byte_offset) & bit;
-			validity.Set(i, valid);
+		for (idx_t i = 0; i < vcount; i++) {
+			ValidityBytes row_mask(validitymask_locations[i]);
+			const auto valid = row_mask.RowIsValid(row_mask.GetValidityEntry(entry_idx), idx_in_entry);
+			const auto col_idx = sel.get_index(i);
+			validity.Set(col_idx, valid);
 		}
 	}
 
@@ -880,55 +877,55 @@ void RowDataCollection::DeserializeIntoVector(Vector &v, const idx_t &vcount, co
 	switch (type) {
 	case PhysicalType::BOOL:
 	case PhysicalType::INT8:
-		TemplatedDeserializeIntoVector<int8_t>(v, vcount, col_idx, key_locations);
+		TemplatedDeserializeIntoVector<int8_t>(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::INT16:
-		TemplatedDeserializeIntoVector<int16_t>(v, vcount, col_idx, key_locations);
+		TemplatedDeserializeIntoVector<int16_t>(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::INT32:
-		TemplatedDeserializeIntoVector<int32_t>(v, vcount, col_idx, key_locations);
+		TemplatedDeserializeIntoVector<int32_t>(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::INT64:
-		TemplatedDeserializeIntoVector<int64_t>(v, vcount, col_idx, key_locations);
+		TemplatedDeserializeIntoVector<int64_t>(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::UINT8:
-		TemplatedDeserializeIntoVector<uint8_t>(v, vcount, col_idx, key_locations);
+		TemplatedDeserializeIntoVector<uint8_t>(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::UINT16:
-		TemplatedDeserializeIntoVector<uint16_t>(v, vcount, col_idx, key_locations);
+		TemplatedDeserializeIntoVector<uint16_t>(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::UINT32:
-		TemplatedDeserializeIntoVector<uint32_t>(v, vcount, col_idx, key_locations);
+		TemplatedDeserializeIntoVector<uint32_t>(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::UINT64:
-		TemplatedDeserializeIntoVector<uint64_t>(v, vcount, col_idx, key_locations);
+		TemplatedDeserializeIntoVector<uint64_t>(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::INT128:
-		TemplatedDeserializeIntoVector<hugeint_t>(v, vcount, col_idx, key_locations);
+		TemplatedDeserializeIntoVector<hugeint_t>(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::FLOAT:
-		TemplatedDeserializeIntoVector<float>(v, vcount, col_idx, key_locations);
+		TemplatedDeserializeIntoVector<float>(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::DOUBLE:
-		TemplatedDeserializeIntoVector<double>(v, vcount, col_idx, key_locations);
+		TemplatedDeserializeIntoVector<double>(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::HASH:
-		TemplatedDeserializeIntoVector<hash_t>(v, vcount, col_idx, key_locations);
+		TemplatedDeserializeIntoVector<hash_t>(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::INTERVAL:
-		TemplatedDeserializeIntoVector<interval_t>(v, vcount, col_idx, key_locations);
+		TemplatedDeserializeIntoVector<interval_t>(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::VARCHAR:
-		DeserializeIntoStringVector(v, vcount, col_idx, key_locations, validitymask_locations);
+		DeserializeIntoStringVector(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::STRUCT:
-		DeserializeIntoStructVector(v, vcount, col_idx, key_locations, validitymask_locations);
+		DeserializeIntoStructVector(v, vcount, sel, key_locations);
 		break;
 	case PhysicalType::LIST:
-		DeserializeIntoListVector(v, vcount, col_idx, key_locations, validitymask_locations);
+		DeserializeIntoListVector(v, vcount, sel, key_locations);
 		break;
 	default:
-		throw NotImplementedException("FIXME: unimplemented deserialize from row-format");
+		throw NotImplementedException("Unimplemented deserialize from row-format");
 	}
 }
 
