@@ -6,17 +6,18 @@
 #include "duckdb/common/types/time.hpp"
 #include "duckdb/common/string_util.hpp"
 #include "duckdb/common/chrono.hpp"
-
+#include "duckdb/common/operator/multiply.hpp"
 #include <ctime>
 
 namespace duckdb {
 
+static_assert(sizeof(timestamp_t) == sizeof(int64_t), "timestamp_t was padded");
+
 // timestamp/datetime uses 64 bits, high 32 bits for date and low 32 bits for time
 // string format is YYYY-MM-DDThh:mm:ssZ
 // T may be a space
 // Z is optional
 // ISO 8601
-
 timestamp_t Timestamp::FromCString(const char *str, idx_t len) {
 	idx_t pos;
 	date_t date;
@@ -28,7 +29,7 @@ timestamp_t Timestamp::FromCString(const char *str, idx_t len) {
 	}
 	if (pos == len) {
 		// no time: only a date
-		return Timestamp::FromDatetime(date, 0);
+		return Timestamp::FromDatetime(date, dtime_t(0));
 	}
 	// try to parse a time field
 	if (str[pos] == ' ' || str[pos] == 'T') {
@@ -41,11 +42,17 @@ timestamp_t Timestamp::FromCString(const char *str, idx_t len) {
 		                          string(str, len));
 	}
 	pos += time_pos;
+	auto timestamp = Timestamp::FromDatetime(date, time);
 	if (pos < len) {
 		// skip a "Z" at the end (as per the ISO8601 specs)
 		if (str[pos] == 'Z') {
 			pos++;
 		}
+		int hour_offset, minute_offset;
+		if (Timestamp::TryParseUTCOffset(str, pos, len, hour_offset, minute_offset)) {
+			timestamp -= hour_offset * Interval::MICROS_PER_HOUR + minute_offset * Interval::MICROS_PER_MINUTE;
+		}
+
 		// skip any spaces at the end
 		while (pos < len && StringUtil::CharacterIsSpace(str[pos])) {
 			pos++;
@@ -56,7 +63,55 @@ timestamp_t Timestamp::FromCString(const char *str, idx_t len) {
 			                          string(str, len));
 		}
 	}
-	return Timestamp::FromDatetime(date, time);
+	return timestamp;
+}
+
+bool Timestamp::TryParseUTCOffset(const char *str, idx_t &pos, idx_t len, int &hour_offset, int &minute_offset) {
+	minute_offset = 0;
+	idx_t curpos = pos;
+	// parse the next 3 characters
+	if (curpos + 3 > len) {
+		// no characters left to parse
+		return false;
+	}
+	char sign_char = str[curpos];
+	if (sign_char != '+' && sign_char != '-') {
+		// expected either + or -
+		return false;
+	}
+	curpos++;
+	if (!StringUtil::CharacterIsDigit(str[curpos]) || !StringUtil::CharacterIsDigit(str[curpos + 1])) {
+		// expected +HH or -HH
+		return false;
+	}
+	hour_offset = (str[curpos] - '0') * 10 + (str[curpos + 1] - '0');
+	if (sign_char == '-') {
+		hour_offset = -hour_offset;
+	}
+	curpos += 2;
+
+	// optional minute specifier: expected either "MM" or ":MM"
+	if (curpos >= len) {
+		// done, nothing left
+		pos = curpos;
+		return true;
+	}
+	if (str[curpos] == ':') {
+		curpos++;
+	}
+	if (curpos + 2 > len || !StringUtil::CharacterIsDigit(str[curpos]) ||
+	    !StringUtil::CharacterIsDigit(str[curpos + 1])) {
+		// no MM specifier
+		pos = curpos;
+		return true;
+	}
+	// we have an MM specifier: parse it
+	minute_offset = (str[curpos] - '0') * 10 + (str[curpos + 1] - '0');
+	if (sign_char == '-') {
+		minute_offset = -minute_offset;
+	}
+	pos = curpos + 2;
+	return true;
 }
 
 timestamp_t Timestamp::FromString(const string &str) {
@@ -71,21 +126,21 @@ string Timestamp::ToString(timestamp_t timestamp) {
 }
 
 date_t Timestamp::GetDate(timestamp_t timestamp) {
-	return (timestamp + (timestamp < 0)) / Interval::MICROS_PER_DAY - (timestamp < 0);
+	return date_t((timestamp.value + (timestamp.value < 0)) / Interval::MICROS_PER_DAY - (timestamp.value < 0));
 }
 
 dtime_t Timestamp::GetTime(timestamp_t timestamp) {
 	date_t date = Timestamp::GetDate(timestamp);
-	return timestamp - (int64_t(date) * int64_t(Interval::MICROS_PER_DAY));
+	return dtime_t(timestamp.value - (int64_t(date.days) * int64_t(Interval::MICROS_PER_DAY)));
 }
 
 timestamp_t Timestamp::FromDatetime(date_t date, dtime_t time) {
-	return date * Interval::MICROS_PER_DAY + time;
+	return timestamp_t(date.days * Interval::MICROS_PER_DAY + time.micros);
 }
 
 void Timestamp::Convert(timestamp_t timestamp, date_t &out_date, dtime_t &out_time) {
 	out_date = GetDate(timestamp);
-	out_time = timestamp - (int64_t(out_date) * int64_t(Interval::MICROS_PER_DAY));
+	out_time = dtime_t(timestamp.value - (int64_t(out_date.days) * int64_t(Interval::MICROS_PER_DAY)));
 	D_ASSERT(timestamp == Timestamp::FromDatetime(out_date, out_time));
 }
 
@@ -96,35 +151,48 @@ timestamp_t Timestamp::GetCurrentTimestamp() {
 }
 
 timestamp_t Timestamp::FromEpochSeconds(int64_t sec) {
-	return sec * Interval::MICROS_PER_SEC;
+	int64_t result;
+	if (!TryMultiplyOperator::Operation(sec, Interval::MICROS_PER_SEC, result)) {
+		throw ConversionException("Could not convert Timestamp(S) to Timestamp(US)");
+	}
+	return timestamp_t(result);
 }
 
 timestamp_t Timestamp::FromEpochMs(int64_t ms) {
-	return ms * Interval::MICROS_PER_MSEC;
+	int64_t result;
+	if (!TryMultiplyOperator::Operation(ms, Interval::MICROS_PER_MSEC, result)) {
+		throw ConversionException("Could not convert Timestamp(MS) to Timestamp(US)");
+	}
+	return timestamp_t(result);
 }
 
 timestamp_t Timestamp::FromEpochMicroSeconds(int64_t micros) {
-	return micros;
+	return timestamp_t(micros);
 }
 
 timestamp_t Timestamp::FromEpochNanoSeconds(int64_t ns) {
-	return ns / 1000;
+	return timestamp_t(ns / 1000);
 }
 
 int64_t Timestamp::GetEpochSeconds(timestamp_t timestamp) {
-	return timestamp / Interval::MICROS_PER_SEC;
+	return timestamp.value / Interval::MICROS_PER_SEC;
 }
 
 int64_t Timestamp::GetEpochMs(timestamp_t timestamp) {
-	return timestamp / Interval::MICROS_PER_MSEC;
+	return timestamp.value / Interval::MICROS_PER_MSEC;
 }
 
 int64_t Timestamp::GetEpochMicroSeconds(timestamp_t timestamp) {
-	return timestamp;
+	return timestamp.value;
 }
 
 int64_t Timestamp::GetEpochNanoSeconds(timestamp_t timestamp) {
-	return timestamp * 1000;
+	int64_t result;
+	int64_t ns_in_us = 1000;
+	if (!TryMultiplyOperator::Operation(timestamp.value, ns_in_us, result)) {
+		throw ConversionException("Could not convert Timestamp(US) to Timestamp(NS)");
+	}
+	return result;
 }
 
 } // namespace duckdb
