@@ -49,14 +49,6 @@ void VectorListBuffer::Reserve(const Vector &to_append, idx_t to_reserve) {
 	if (to_reserve > capacity) {
 		idx_t new_capacity = (to_reserve) / STANDARD_VECTOR_SIZE + ((to_reserve) % STANDARD_VECTOR_SIZE != 0);
 		new_capacity *= STANDARD_VECTOR_SIZE;
-		if (child->GetType().id() == LogicalTypeId::STRUCT && size == 0) {
-			// Empty struct, gotta initialize it first
-			auto &source_children = StructVector::GetEntries(to_append);
-			for (auto &src_child : source_children) {
-				auto child_copy = make_unique<Vector>(src_child.second->GetType());
-				StructVector::AddEntry(*child, src_child.first, move(child_copy));
-			}
-		}
 		child->Resize(capacity, new_capacity);
 		capacity = new_capacity;
 	}
