@@ -9,51 +9,51 @@
 
 namespace duckdb {
 
-template <bool INVERSE> void is_null_loop(Vector &input, Vector &result, idx_t count) {
-	D_ASSERT(result.type == LogicalType::BOOLEAN);
+template <bool INVERSE>
+void IsNullLoop(Vector &input, Vector &result, idx_t count) {
+	D_ASSERT(result.GetType() == LogicalType::BOOLEAN);
 
-	if (input.vector_type == VectorType::CONSTANT_VECTOR) {
-		result.vector_type = VectorType::CONSTANT_VECTOR;
+	if (input.GetVectorType() == VectorType::CONSTANT_VECTOR) {
+		result.SetVectorType(VectorType::CONSTANT_VECTOR);
 		auto result_data = ConstantVector::GetData<bool>(result);
 		*result_data = INVERSE ? !ConstantVector::IsNull(input) : ConstantVector::IsNull(input);
 	} else {
 		VectorData data;
 		input.Orrify(count, data);
 
-		result.vector_type = VectorType::FLAT_VECTOR;
+		result.SetVectorType(VectorType::FLAT_VECTOR);
 		auto result_data = FlatVector::GetData<bool>(result);
-		auto &nullmask = *data.nullmask;
 		for (idx_t i = 0; i < count; i++) {
 			auto idx = data.sel->get_index(i);
-			result_data[i] = INVERSE ? !nullmask[idx] : nullmask[idx];
+			result_data[i] = INVERSE ? data.validity.RowIsValid(idx) : !data.validity.RowIsValid(idx);
 		}
 	}
 }
 
 void VectorOperations::IsNotNull(Vector &input, Vector &result, idx_t count) {
-	is_null_loop<true>(input, result, count);
+	IsNullLoop<true>(input, result, count);
 }
 
 void VectorOperations::IsNull(Vector &input, Vector &result, idx_t count) {
-	is_null_loop<false>(input, result, count);
+	IsNullLoop<false>(input, result, count);
 }
 
 bool VectorOperations::HasNotNull(Vector &input, idx_t count) {
 	if (count == 0) {
 		return false;
 	}
-	if (input.vector_type == VectorType::CONSTANT_VECTOR) {
+	if (input.GetVectorType() == VectorType::CONSTANT_VECTOR) {
 		return !ConstantVector::IsNull(input);
 	} else {
 		VectorData data;
 		input.Orrify(count, data);
 
-		if (data.nullmask->none()) {
+		if (data.validity.AllValid()) {
 			return true;
 		}
 		for (idx_t i = 0; i < count; i++) {
 			auto idx = data.sel->get_index(i);
-			if (!(*data.nullmask)[idx]) {
+			if (data.validity.RowIsValid(idx)) {
 				return true;
 			}
 		}
@@ -65,18 +65,18 @@ bool VectorOperations::HasNull(Vector &input, idx_t count) {
 	if (count == 0) {
 		return false;
 	}
-	if (input.vector_type == VectorType::CONSTANT_VECTOR) {
+	if (input.GetVectorType() == VectorType::CONSTANT_VECTOR) {
 		return ConstantVector::IsNull(input);
 	} else {
 		VectorData data;
 		input.Orrify(count, data);
 
-		if (data.nullmask->none()) {
+		if (data.validity.AllValid()) {
 			return false;
 		}
 		for (idx_t i = 0; i < count; i++) {
 			auto idx = data.sel->get_index(i);
-			if ((*data.nullmask)[idx]) {
+			if (!data.validity.RowIsValid(idx)) {
 				return true;
 			}
 		}
