@@ -3,6 +3,8 @@
 #include "duckdb/common/algorithm.hpp"
 #include "duckdb/common/exception.hpp"
 #include "duckdb/common/types/null_value.hpp"
+#include "duckdb/common/types/row_data_collection.hpp"
+#include "duckdb/common/row_operations/row_operations.hpp"
 #include "duckdb/common/vector_operations/vector_operations.hpp"
 #include "duckdb/execution/expression_executor.hpp"
 #include "duckdb/planner/expression/bound_aggregate_expression.hpp"
@@ -17,6 +19,8 @@
 
 namespace duckdb {
 
+using ValidityBytes = RowLayout::ValidityBytes;
+
 GroupedAggregateHashTable::GroupedAggregateHashTable(BufferManager &buffer_manager, vector<LogicalType> group_types,
                                                      vector<LogicalType> payload_types,
                                                      const vector<BoundAggregateExpression *> &bindings,
@@ -33,17 +37,19 @@ GroupedAggregateHashTable::GroupedAggregateHashTable(BufferManager &buffer_manag
                                                      vector<LogicalType> payload_types_p,
                                                      vector<AggregateObject> aggregate_objects_p,
                                                      HtEntryType entry_type)
-    : BaseAggregateHashTable(buffer_manager, move(group_types_p), move(payload_types_p), move(aggregate_objects_p)),
-      entry_type(entry_type), capacity(0), entries(0), payload_page_offset(0), is_finalized(false),
-      ht_offsets(LogicalTypeId::BIGINT), hash_salts(LogicalTypeId::SMALLINT),
-      group_compare_vector(STANDARD_VECTOR_SIZE), no_match_vector(STANDARD_VECTOR_SIZE),
-      empty_vector(STANDARD_VECTOR_SIZE) {
+    : BaseAggregateHashTable(buffer_manager, move(payload_types_p)), entry_type(entry_type), capacity(0), entries(0),
+      payload_page_offset(0), is_finalized(false), ht_offsets(LogicalTypeId::BIGINT),
+      hash_salts(LogicalTypeId::SMALLINT), group_compare_vector(STANDARD_VECTOR_SIZE),
+      no_match_vector(STANDARD_VECTOR_SIZE), empty_vector(STANDARD_VECTOR_SIZE) {
+
+	// Append hash column to the end and initialise the row layout
+	group_types_p.emplace_back(LogicalType::HASH);
+	layout.Initialize(move(group_types_p), move(aggregate_objects_p));
 
 	// HT layout
-	tuple_size = HASH_WIDTH + group_width + payload_width;
-#ifndef DUCKDB_ALLOW_UNDEFINED
-	tuple_size = BaseAggregateHashTable::Align(tuple_size);
-#endif
+	hash_offset = layout.GetOffsets()[layout.ColumnCount() - 1];
+
+	tuple_size = layout.GetRowWidth();
 
 	D_ASSERT(tuple_size <= Storage::BLOCK_ALLOC_SIZE);
 	tuples_per_block = Storage::BLOCK_ALLOC_SIZE / tuple_size;
@@ -66,14 +72,16 @@ GroupedAggregateHashTable::GroupedAggregateHashTable(BufferManager &buffer_manag
 	}
 
 	// create additional hash tables for distinct aggrs
+	auto &aggregates = layout.GetAggregates();
 	distinct_hashes.resize(aggregates.size());
 
 	idx_t payload_idx = 0;
 	for (idx_t i = 0; i < aggregates.size(); i++) {
 		auto &aggr = aggregates[i];
 		if (aggr.distinct) {
-			// group types plus aggr return type
-			vector<LogicalType> distinct_group_types(group_types);
+			// layout types minus hash column plus aggr return type
+			vector<LogicalType> distinct_group_types(layout.GetTypes());
+			(void)distinct_group_types.pop_back();
 			for (idx_t child_idx = 0; child_idx < aggr.child_count; child_idx++) {
 				distinct_group_types.push_back(payload_types[payload_idx]);
 			}
@@ -82,6 +90,8 @@ GroupedAggregateHashTable::GroupedAggregateHashTable(BufferManager &buffer_manag
 		payload_idx += aggr.child_count;
 	}
 	addresses.Initialize(LogicalType::POINTER);
+	predicates.resize(layout.ColumnCount() - 1, ExpressionType::COMPARE_EQUAL);
+	string_heap = make_unique<RowDataCollection>(buffer_manager, Storage::BLOCK_ALLOC_SIZE / 8, 8);
 }
 
 GroupedAggregateHashTable::~GroupedAggregateHashTable() {
@@ -120,8 +130,8 @@ void GroupedAggregateHashTable::NewBlock() {
 void GroupedAggregateHashTable::Destroy() {
 	// check if there is a destructor
 	bool has_destructor = false;
-	for (idx_t i = 0; i < aggregates.size(); i++) {
-		if (aggregates[i].function.destructor) {
+	for (auto &aggr : layout.GetAggregates()) {
+		if (aggr.function.destructor) {
 			has_destructor = true;
 		}
 	}
@@ -135,18 +145,18 @@ void GroupedAggregateHashTable::Destroy() {
 	idx_t count = 0;
 
 	PayloadApply([&](idx_t page_nr, idx_t page_offset, data_ptr_t ptr) {
-		data_pointers[count++] = ptr + HASH_WIDTH + group_width;
+		data_pointers[count++] = ptr;
 		if (count == STANDARD_VECTOR_SIZE) {
-			CallDestructors(state_vector, count);
+			RowOperations::DestroyStates(layout, state_vector, count);
 			count = 0;
 		}
 	});
-	CallDestructors(state_vector, count);
+	RowOperations::DestroyStates(layout, state_vector, count);
 }
 
-template <class T>
+template <class ENTRY>
 void GroupedAggregateHashTable::VerifyInternal() {
-	auto hashes_ptr = (T *)hashes_hdl_ptr;
+	auto hashes_ptr = (ENTRY *)hashes_hdl_ptr;
 	D_ASSERT(payload_hds.size() == payload_hds_ptrs.size());
 	idx_t count = 0;
 	for (idx_t i = 0; i < capacity; i++) {
@@ -154,7 +164,7 @@ void GroupedAggregateHashTable::VerifyInternal() {
 			D_ASSERT(hashes_ptr[i].page_offset < tuples_per_block);
 			D_ASSERT(hashes_ptr[i].page_nr <= payload_hds.size());
 			auto ptr = payload_hds_ptrs[hashes_ptr[i].page_nr - 1] + ((hashes_ptr[i].page_offset) * tuple_size);
-			auto hash = Load<hash_t>(ptr);
+			auto hash = Load<hash_t>(ptr + hash_offset);
 			D_ASSERT((hashes_ptr[i].salt) == (hash >> hash_prefix_shift));
 
 			count++;
@@ -195,7 +205,7 @@ void GroupedAggregateHashTable::Verify() {
 #endif
 }
 
-template <class T>
+template <class ENTRY>
 void GroupedAggregateHashTable::Resize(idx_t size) {
 	Verify();
 
@@ -212,7 +222,7 @@ void GroupedAggregateHashTable::Resize(idx_t size) {
 	D_ASSERT((size & (size - 1)) == 0);
 	bitmask = size - 1;
 
-	auto byte_size = size * sizeof(T);
+	auto byte_size = size * sizeof(ENTRY);
 	if (byte_size > (idx_t)Storage::BLOCK_ALLOC_SIZE) {
 		hashes_hdl = buffer_manager.Allocate(byte_size);
 		hashes_hdl_ptr = hashes_hdl->Ptr();
@@ -221,10 +231,10 @@ void GroupedAggregateHashTable::Resize(idx_t size) {
 	hashes_end_ptr = hashes_hdl_ptr + byte_size;
 	capacity = size;
 
-	auto hashes_arr = (T *)hashes_hdl_ptr;
+	auto hashes_arr = (ENTRY *)hashes_hdl_ptr;
 
 	PayloadApply([&](idx_t page_nr, idx_t page_offset, data_ptr_t ptr) {
-		auto hash = Load<hash_t>(ptr);
+		auto hash = Load<hash_t>(ptr + hash_offset);
 		D_ASSERT((hash & bitmask) == (hash % capacity));
 		auto entry_idx = (idx_t)hash & bitmask;
 		while (hashes_arr[entry_idx].page_nr > 0) {
@@ -252,22 +262,6 @@ idx_t GroupedAggregateHashTable::AddChunk(DataChunk &groups, DataChunk &payload)
 	return AddChunk(groups, hashes, payload);
 }
 
-void GroupedAggregateHashTable::UpdateAggregate(AggregateObject &aggr, DataChunk &payload, Vector &distinct_addresses,
-                                                idx_t input_count, idx_t payload_idx) {
-	ExpressionExecutor filter_execution(aggr.filter);
-	SelectionVector true_sel(STANDARD_VECTOR_SIZE);
-	auto count = filter_execution.SelectExpression(payload, true_sel);
-	DataChunk filtered_payload;
-	auto pay_types = payload.GetTypes();
-	filtered_payload.Initialize(pay_types);
-	filtered_payload.Slice(payload, true_sel, count);
-	Vector filtered_addresses;
-	filtered_addresses.Slice(distinct_addresses, true_sel, count);
-	filtered_addresses.Normalify(count);
-	aggr.function.update(input_count == 0 ? nullptr : &filtered_payload.data[payload_idx], nullptr, input_count,
-	                     filtered_addresses, filtered_payload.size());
-}
-
 idx_t GroupedAggregateHashTable::AddChunk(DataChunk &groups, Vector &group_hashes, DataChunk &payload) {
 	D_ASSERT(!is_finalized);
 
@@ -277,37 +271,36 @@ idx_t GroupedAggregateHashTable::AddChunk(DataChunk &groups, Vector &group_hashe
 	// dummy
 	SelectionVector new_groups(STANDARD_VECTOR_SIZE);
 
-	D_ASSERT(groups.ColumnCount() == group_types.size());
-#ifdef DEBUG
-	for (idx_t i = 0; i < group_types.size(); i++) {
-		D_ASSERT(groups.GetTypes()[i] == group_types[i]);
+	D_ASSERT(groups.ColumnCount() + 1 == layout.ColumnCount());
+	for (idx_t i = 0; i < groups.ColumnCount(); i++) {
+		D_ASSERT(groups.GetTypes()[i] == layout.GetTypes()[i]);
 	}
-#endif
 
 	Vector addresses(LogicalType::POINTER);
 	auto new_group_count = FindOrCreateGroups(groups, group_hashes, addresses, new_groups);
+	VectorOperations::AddInPlace(addresses, layout.GetAggrOffset(), payload.size());
 
 	// now every cell has an entry
 	// update the aggregates
 	idx_t payload_idx = 0;
 
+	auto &aggregates = layout.GetAggregates();
 	for (idx_t aggr_idx = 0; aggr_idx < aggregates.size(); aggr_idx++) {
 		// for any entries for which a group was found, update the aggregate
 		auto &aggr = aggregates[aggr_idx];
-		auto input_count = (idx_t)aggr.child_count;
 		if (aggr.distinct) {
 			// construct chunk for secondary hash table probing
-			vector<LogicalType> probe_types(group_types);
+			vector<LogicalType> probe_types(groups.GetTypes());
 			for (idx_t i = 0; i < aggr.child_count; i++) {
 				probe_types.push_back(payload_types[payload_idx]);
 			}
 			DataChunk probe_chunk;
 			probe_chunk.Initialize(probe_types);
-			for (idx_t group_idx = 0; group_idx < group_types.size(); group_idx++) {
+			for (idx_t group_idx = 0; group_idx < groups.ColumnCount(); group_idx++) {
 				probe_chunk.data[group_idx].Reference(groups.data[group_idx]);
 			}
 			for (idx_t i = 0; i < aggr.child_count; i++) {
-				probe_chunk.data[group_types.size() + i].Reference(payload.data[payload_idx + i]);
+				probe_chunk.data[groups.ColumnCount() + i].Reference(payload.data[payload_idx + i]);
 			}
 			probe_chunk.SetCardinality(groups);
 			probe_chunk.Verify();
@@ -330,7 +323,7 @@ idx_t GroupedAggregateHashTable::AddChunk(DataChunk &groups, Vector &group_hashe
 					distinct_payload.Slice(payload, new_groups, new_group_count);
 					distinct_addresses.Verify(new_group_count);
 					distinct_addresses.Normalify(new_group_count);
-					UpdateAggregate(aggr, distinct_payload, distinct_addresses, input_count, payload_idx);
+					RowOperations::UpdateFilteredStates(aggr, distinct_addresses, distinct_payload, payload_idx);
 				} else {
 					Vector distinct_addresses;
 					distinct_addresses.Slice(addresses, new_groups, new_group_count);
@@ -340,19 +333,17 @@ idx_t GroupedAggregateHashTable::AddChunk(DataChunk &groups, Vector &group_hashe
 					}
 					distinct_addresses.Verify(new_group_count);
 
-					aggr.function.update(input_count == 0 ? nullptr : &payload.data[payload_idx], nullptr, input_count,
-					                     distinct_addresses, new_group_count);
+					RowOperations::UpdateStates(aggr, distinct_addresses, payload, payload_idx, new_group_count);
 				}
 			}
 		} else if (aggr.filter) {
-			UpdateAggregate(aggr, payload, addresses, input_count, payload_idx);
+			RowOperations::UpdateFilteredStates(aggr, addresses, payload, payload_idx);
 		} else {
-			aggr.function.update(input_count == 0 ? nullptr : &payload.data[payload_idx], aggr.bind_data, input_count,
-			                     addresses, payload.size());
+			RowOperations::UpdateStates(aggr, addresses, payload, payload_idx, payload.size());
 		}
 
 		// move to the next aggregate
-		payload_idx += input_count;
+		payload_idx += aggr.child_count;
 		VectorOperations::AddInPlace(addresses, aggr.payload_size, payload.size());
 	}
 
@@ -362,7 +353,7 @@ idx_t GroupedAggregateHashTable::AddChunk(DataChunk &groups, Vector &group_hashe
 
 void GroupedAggregateHashTable::FetchAggregates(DataChunk &groups, DataChunk &result) {
 	groups.Verify();
-	D_ASSERT(groups.ColumnCount() == group_types.size());
+	D_ASSERT(groups.ColumnCount() + 1 == layout.ColumnCount());
 	for (idx_t i = 0; i < result.ColumnCount(); i++) {
 		D_ASSERT(result.data[i].GetType() == payload_types[i]);
 	}
@@ -375,217 +366,10 @@ void GroupedAggregateHashTable::FetchAggregates(DataChunk &groups, DataChunk &re
 	Vector addresses(LogicalType::POINTER);
 	FindOrCreateGroups(groups, addresses);
 	// now fetch the aggregates
-	for (idx_t aggr_idx = 0; aggr_idx < aggregates.size(); aggr_idx++) {
-		D_ASSERT(result.ColumnCount() > aggr_idx);
-
-		VectorOperations::Gather::Set(addresses, result.data[aggr_idx], groups.size());
-	}
+	RowOperations::FinalizeStates(layout, addresses, result, 0);
 }
 
-template <class T>
-static void TemplatedScatter(VectorData &gdata, Vector &addresses, const SelectionVector &sel, idx_t count,
-                             idx_t type_size) {
-	auto data = (T *)gdata.data;
-	auto pointers = FlatVector::GetData<uintptr_t>(addresses);
-	if (!gdata.validity.AllValid()) {
-		for (idx_t i = 0; i < count; i++) {
-			auto pointer_idx = sel.get_index(i);
-			auto group_idx = gdata.sel->get_index(pointer_idx);
-			auto ptr = (T *)pointers[pointer_idx];
-
-			T store_value = !gdata.validity.RowIsValid(group_idx) ? NullValue<T>() : data[group_idx];
-			Store<T>(store_value, (data_ptr_t)ptr);
-			pointers[pointer_idx] += type_size;
-		}
-	} else {
-		for (idx_t i = 0; i < count; i++) {
-			auto pointer_idx = sel.get_index(i);
-			auto group_idx = gdata.sel->get_index(pointer_idx);
-			auto ptr = (T *)pointers[pointer_idx];
-
-			Store<T>(data[group_idx], (data_ptr_t)ptr);
-			pointers[pointer_idx] += type_size;
-		}
-	}
-}
-
-void GroupedAggregateHashTable::ScatterGroups(DataChunk &groups, unique_ptr<VectorData[]> &group_data,
-                                              Vector &addresses, const SelectionVector &sel, idx_t count) {
-	if (count == 0) {
-		return;
-	}
-	for (idx_t grp_idx = 0; grp_idx < groups.ColumnCount(); grp_idx++) {
-		auto &data = groups.data[grp_idx];
-		auto &gdata = group_data[grp_idx];
-
-		auto type_size = GetTypeIdSize(data.GetType().InternalType());
-
-		switch (data.GetType().InternalType()) {
-		case PhysicalType::BOOL:
-		case PhysicalType::INT8:
-			TemplatedScatter<int8_t>(gdata, addresses, sel, count, type_size);
-			break;
-		case PhysicalType::INT16:
-			TemplatedScatter<int16_t>(gdata, addresses, sel, count, type_size);
-			break;
-		case PhysicalType::INT32:
-			TemplatedScatter<int32_t>(gdata, addresses, sel, count, type_size);
-			break;
-		case PhysicalType::INT64:
-			TemplatedScatter<int64_t>(gdata, addresses, sel, count, type_size);
-			break;
-		case PhysicalType::UINT8:
-			TemplatedScatter<uint8_t>(gdata, addresses, sel, count, type_size);
-			break;
-		case PhysicalType::UINT16:
-			TemplatedScatter<uint16_t>(gdata, addresses, sel, count, type_size);
-			break;
-		case PhysicalType::UINT32:
-			TemplatedScatter<uint32_t>(gdata, addresses, sel, count, type_size);
-			break;
-		case PhysicalType::UINT64:
-			TemplatedScatter<uint64_t>(gdata, addresses, sel, count, type_size);
-			break;
-		case PhysicalType::INT128:
-			TemplatedScatter<hugeint_t>(gdata, addresses, sel, count, type_size);
-			break;
-		case PhysicalType::FLOAT:
-			TemplatedScatter<float>(gdata, addresses, sel, count, type_size);
-			break;
-		case PhysicalType::DOUBLE:
-			TemplatedScatter<double>(gdata, addresses, sel, count, type_size);
-			break;
-		case PhysicalType::INTERVAL:
-			TemplatedScatter<interval_t>(gdata, addresses, sel, count, type_size);
-			break;
-		case PhysicalType::VARCHAR: {
-			auto string_data = (string_t *)gdata.data;
-			auto pointers = FlatVector::GetData<data_ptr_t>(addresses);
-
-			for (idx_t i = 0; i < count; i++) {
-				auto pointer_idx = sel.get_index(i);
-				auto group_idx = gdata.sel->get_index(pointer_idx);
-				auto ptr = pointers[pointer_idx];
-				if (!gdata.validity.RowIsValid(group_idx)) {
-					Store<string_t>(NullValue<string_t>(), ptr);
-				} else if (string_data[group_idx].IsInlined()) {
-					Store<string_t>(string_data[group_idx], ptr);
-				} else {
-					Store<string_t>(
-					    string_heap.AddBlob(string_data[group_idx].GetDataUnsafe(), string_data[group_idx].GetSize()),
-					    ptr);
-				}
-
-				pointers[pointer_idx] += type_size;
-			}
-			break;
-		}
-		default:
-			throw Exception("Unsupported type for group vector");
-		}
-	}
-}
-
-template <class T>
-static void TemplatedCompareGroups(VectorData &gdata, Vector &addresses, SelectionVector &sel, idx_t &count,
-                                   idx_t type_size, SelectionVector &no_match, idx_t &no_match_count) {
-	auto data = (T *)gdata.data;
-	auto pointers = FlatVector::GetData<uintptr_t>(addresses);
-	idx_t match_count = 0;
-	if (!gdata.validity.AllValid()) {
-		for (idx_t i = 0; i < count; i++) {
-			auto idx = sel.get_index(i);
-			auto group_idx = gdata.sel->get_index(idx);
-			auto value = Load<T>((data_ptr_t)pointers[idx]);
-
-			if (!gdata.validity.RowIsValid(group_idx)) {
-				if (IsNullValue<T>(value)) {
-					// match: move to next value to compare
-					sel.set_index(match_count++, idx);
-					pointers[idx] += type_size;
-				} else {
-					no_match.set_index(no_match_count++, idx);
-				}
-			} else {
-				if (Equals::Operation<T>(data[group_idx], value)) {
-					sel.set_index(match_count++, idx);
-					pointers[idx] += type_size;
-				} else {
-					no_match.set_index(no_match_count++, idx);
-				}
-			}
-		}
-	} else {
-		for (idx_t i = 0; i < count; i++) {
-			auto idx = sel.get_index(i);
-			auto group_idx = gdata.sel->get_index(idx);
-			auto value = Load<T>((data_ptr_t)pointers[idx]);
-
-			if (Equals::Operation<T>(data[group_idx], value)) {
-				sel.set_index(match_count++, idx);
-				pointers[idx] += type_size;
-			} else {
-				no_match.set_index(no_match_count++, idx);
-			}
-		}
-	}
-	count = match_count;
-}
-
-static void CompareGroups(DataChunk &groups, unique_ptr<VectorData[]> &group_data, Vector &addresses,
-                          SelectionVector &sel, idx_t count, SelectionVector &no_match, idx_t &no_match_count) {
-	for (idx_t group_idx = 0; group_idx < groups.ColumnCount(); group_idx++) {
-		auto &data = groups.data[group_idx];
-		auto &gdata = group_data[group_idx];
-		auto type_size = GetTypeIdSize(data.GetType().InternalType());
-		switch (data.GetType().InternalType()) {
-		case PhysicalType::BOOL:
-		case PhysicalType::INT8:
-			TemplatedCompareGroups<int8_t>(gdata, addresses, sel, count, type_size, no_match, no_match_count);
-			break;
-		case PhysicalType::INT16:
-			TemplatedCompareGroups<int16_t>(gdata, addresses, sel, count, type_size, no_match, no_match_count);
-			break;
-		case PhysicalType::INT32:
-			TemplatedCompareGroups<int32_t>(gdata, addresses, sel, count, type_size, no_match, no_match_count);
-			break;
-		case PhysicalType::INT64:
-			TemplatedCompareGroups<int64_t>(gdata, addresses, sel, count, type_size, no_match, no_match_count);
-			break;
-		case PhysicalType::UINT8:
-			TemplatedCompareGroups<uint8_t>(gdata, addresses, sel, count, type_size, no_match, no_match_count);
-			break;
-		case PhysicalType::UINT16:
-			TemplatedCompareGroups<uint16_t>(gdata, addresses, sel, count, type_size, no_match, no_match_count);
-			break;
-		case PhysicalType::UINT32:
-			TemplatedCompareGroups<uint32_t>(gdata, addresses, sel, count, type_size, no_match, no_match_count);
-			break;
-		case PhysicalType::UINT64:
-			TemplatedCompareGroups<uint64_t>(gdata, addresses, sel, count, type_size, no_match, no_match_count);
-			break;
-		case PhysicalType::INT128:
-			TemplatedCompareGroups<hugeint_t>(gdata, addresses, sel, count, type_size, no_match, no_match_count);
-			break;
-		case PhysicalType::FLOAT:
-			TemplatedCompareGroups<float>(gdata, addresses, sel, count, type_size, no_match, no_match_count);
-			break;
-		case PhysicalType::DOUBLE:
-			TemplatedCompareGroups<double>(gdata, addresses, sel, count, type_size, no_match, no_match_count);
-			break;
-		case PhysicalType::INTERVAL:
-			TemplatedCompareGroups<interval_t>(gdata, addresses, sel, count, type_size, no_match, no_match_count);
-			break;
-		case PhysicalType::VARCHAR:
-			TemplatedCompareGroups<string_t>(gdata, addresses, sel, count, type_size, no_match, no_match_count);
-			break;
-		default:
-			throw Exception("Unsupported type for group vector");
-		}
-	}
-}
-
-template <class T>
+template <class ENTRY>
 idx_t GroupedAggregateHashTable::FindOrCreateGroupsInternal(DataChunk &groups, Vector &group_hashes, Vector &addresses,
                                                             SelectionVector &new_groups_out) {
 
@@ -597,11 +381,11 @@ idx_t GroupedAggregateHashTable::FindOrCreateGroupsInternal(DataChunk &groups, V
 
 	// resize at 50% capacity, also need to fit the entire vector
 	if (capacity - entries <= groups.size() || entries > capacity / LOAD_FACTOR) {
-		Resize<T>(capacity * 2);
+		Resize<ENTRY>(capacity * 2);
 	}
 
 	D_ASSERT(capacity - entries >= groups.size());
-	D_ASSERT(groups.ColumnCount() == group_types.size());
+	D_ASSERT(groups.ColumnCount() + 1 == layout.ColumnCount());
 	// we need to be able to fit at least one vector of data
 	D_ASSERT(capacity - entries >= groups.size());
 	D_ASSERT(group_hashes.GetType() == LogicalType::HASH);
@@ -634,11 +418,17 @@ idx_t GroupedAggregateHashTable::FindOrCreateGroupsInternal(DataChunk &groups, V
 
 	idx_t remaining_entries = groups.size();
 
-	// orrify all the groups
-	auto group_data = unique_ptr<VectorData[]>(new VectorData[groups.ColumnCount()]);
+	// make a chunk that references the groups and the hashes
+	DataChunk group_chunk;
+	group_chunk.InitializeEmpty(layout.GetTypes());
 	for (idx_t grp_idx = 0; grp_idx < groups.ColumnCount(); grp_idx++) {
-		groups.data[grp_idx].Orrify(groups.size(), group_data[grp_idx]);
+		group_chunk.data[grp_idx].Reference(groups.data[grp_idx]);
 	}
+	group_chunk.data[groups.ColumnCount()].Reference(group_hashes);
+	group_chunk.SetCardinality(groups);
+
+	// orrify all the groups
+	auto group_data = group_chunk.Orrify();
 
 	idx_t new_group_count = 0;
 	while (remaining_entries > 0) {
@@ -649,7 +439,7 @@ idx_t GroupedAggregateHashTable::FindOrCreateGroupsInternal(DataChunk &groups, V
 		// first figure out for each remaining whether or not it belongs to a full or empty group
 		for (idx_t i = 0; i < remaining_entries; i++) {
 			const idx_t index = sel_vector->get_index(i);
-			const auto ht_entry_ptr = ((T *)this->hashes_hdl_ptr) + ht_offsets_ptr[index];
+			const auto ht_entry_ptr = ((ENTRY *)this->hashes_hdl_ptr) + ht_offsets_ptr[index];
 			if (ht_entry_ptr->page_nr == 0) { // we use page number 0 as a "unused marker"
 				// cell is empty; setup the new entry
 				if (payload_page_offset == tuples_per_block || payload_hds.empty()) {
@@ -658,13 +448,6 @@ idx_t GroupedAggregateHashTable::FindOrCreateGroupsInternal(DataChunk &groups, V
 
 				auto entry_payload_ptr = payload_hds_ptrs.back() + (payload_page_offset * tuple_size);
 
-				// copy the group hash to the payload for use in resize
-				memcpy(entry_payload_ptr, &group_hashes_ptr[index], HASH_WIDTH);
-				D_ASSERT((*(hash_t *)entry_payload_ptr) == group_hashes_ptr[index]);
-
-				// initialize the payload info for the column
-				memcpy(entry_payload_ptr + HASH_WIDTH + group_width, empty_payload_data.get(), payload_width);
-
 				D_ASSERT(group_hashes_ptr[index] >> hash_prefix_shift <= NumericLimits<uint16_t>::Maximum());
 				D_ASSERT(payload_page_offset < tuples_per_block);
 				D_ASSERT(payload_hds.size() < NumericLimits<uint32_t>::Maximum());
@@ -672,8 +455,6 @@ idx_t GroupedAggregateHashTable::FindOrCreateGroupsInternal(DataChunk &groups, V
 
 				ht_entry_ptr->salt = group_hashes_ptr[index] >> hash_prefix_shift;
 
-				D_ASSERT(((*(hash_t *)entry_payload_ptr) >> hash_prefix_shift) == ht_entry_ptr->salt);
-
 				// page numbers start at one so we can use 0 as empty flag
 				// GetPtr undoes this
 				ht_entry_ptr->page_nr = payload_hds.size();
@@ -684,7 +465,7 @@ idx_t GroupedAggregateHashTable::FindOrCreateGroupsInternal(DataChunk &groups, V
 				new_groups_out.set_index(new_group_count++, index);
 				entries++;
 
-				addresses_ptr[index] = entry_payload_ptr + HASH_WIDTH;
+				addresses_ptr[index] = entry_payload_ptr;
 
 			} else {
 				// cell is occupied: add to check list
@@ -694,7 +475,7 @@ idx_t GroupedAggregateHashTable::FindOrCreateGroupsInternal(DataChunk &groups, V
 
 					auto page_ptr = payload_hds_ptrs[ht_entry_ptr->page_nr - 1];
 					auto page_offset = ht_entry_ptr->page_offset * tuple_size;
-					addresses_ptr[index] = page_ptr + page_offset + HASH_WIDTH;
+					addresses_ptr[index] = page_ptr + page_offset;
 
 				} else {
 					no_match_vector.set_index(no_match_count++, index);
@@ -702,16 +483,15 @@ idx_t GroupedAggregateHashTable::FindOrCreateGroupsInternal(DataChunk &groups, V
 			}
 		}
 
-		if (new_entry_count > 0) {
-			// for each of the locations that are empty, serialize the group columns to the locations
-			ScatterGroups(groups, group_data, addresses, empty_vector, new_entry_count);
-		}
+		// for each of the locations that are empty, serialize the group columns to the locations
+		RowOperations::Scatter(group_chunk, group_data.get(), layout, addresses, *string_heap, empty_vector,
+		                       new_entry_count);
+		RowOperations::InitializeStates(layout, addresses, empty_vector, new_entry_count);
+
 		// now we have only the tuples remaining that might match to an existing group
 		// start performing comparisons with each of the groups
-		if (need_compare_count > 0) {
-			CompareGroups(groups, group_data, addresses, group_compare_vector, need_compare_count, no_match_vector,
-			              no_match_count);
-		}
+		RowOperations::Match(group_chunk, group_data.get(), layout, addresses, predicates, group_compare_vector,
+		                     need_compare_count, &no_match_vector, no_match_count);
 
 		// each of the entries that do not match we move them to the next entry in the HT
 		for (idx_t i = 0; i < no_match_count; i++) {
@@ -724,9 +504,7 @@ idx_t GroupedAggregateHashTable::FindOrCreateGroupsInternal(DataChunk &groups, V
 		sel_vector = &no_match_vector;
 		remaining_entries = no_match_count;
 	}
-	// pointers in addresses now were moved behind the grousp by CompareGroups/ScatterGroups but we may have to add
-	// padding still to point at the payload.
-	VectorOperations::AddInPlace(addresses, group_padding, groups.size());
+
 	return new_group_count;
 }
 
@@ -762,11 +540,12 @@ void GroupedAggregateHashTable::FlushMove(Vector &source_addresses, Vector &sour
 	D_ASSERT(source_hashes.GetType() == LogicalType::HASH);
 
 	DataChunk groups;
-	groups.Initialize(group_types);
+	groups.Initialize(vector<LogicalType>(layout.GetTypes().begin(), layout.GetTypes().end() - 1));
 	groups.SetCardinality(count);
 	for (idx_t i = 0; i < groups.ColumnCount(); i++) {
 		auto &column = groups.data[i];
-		VectorOperations::Gather::Set(source_addresses, column, count);
+		RowOperations::Gather(layout, source_addresses, FlatVector::INCREMENTAL_SELECTION_VECTOR, column,
+		                      FlatVector::INCREMENTAL_SELECTION_VECTOR, count, i);
 	}
 
 	SelectionVector new_groups(STANDARD_VECTOR_SIZE);
@@ -775,24 +554,16 @@ void GroupedAggregateHashTable::FlushMove(Vector &source_addresses, Vector &sour
 
 	FindOrCreateGroups(groups, source_hashes, group_addresses, new_groups_sel);
 
-	VectorOperations::AddInPlace(source_addresses, group_padding, count);
-
-	for (auto &aggr : aggregates) {
-		// for any entries for which a group was found, update the aggregate
-		D_ASSERT(aggr.function.combine);
-		aggr.function.combine(source_addresses, group_addresses, count);
-		VectorOperations::AddInPlace(source_addresses, aggr.payload_size, count);
-		VectorOperations::AddInPlace(group_addresses, aggr.payload_size, count);
-	}
+	RowOperations::CombineStates(layout, source_addresses, group_addresses, count);
 }
 
 void GroupedAggregateHashTable::Combine(GroupedAggregateHashTable &other) {
 
 	D_ASSERT(!is_finalized);
 
-	D_ASSERT(other.payload_width == payload_width);
-	D_ASSERT(other.group_width == group_width);
-	D_ASSERT(other.tuple_size == tuple_size);
+	D_ASSERT(other.layout.GetAggrWidth() == layout.GetAggrWidth());
+	D_ASSERT(other.layout.GetDataWidth() == layout.GetDataWidth());
+	D_ASSERT(other.layout.GetRowWidth() == layout.GetRowWidth());
 	D_ASSERT(other.tuples_per_block == tuples_per_block);
 
 	if (other.entries == 0) {
@@ -808,10 +579,10 @@ void GroupedAggregateHashTable::Combine(GroupedAggregateHashTable &other) {
 	idx_t group_idx = 0;
 
 	other.PayloadApply([&](idx_t page_nr, idx_t page_offset, data_ptr_t ptr) {
-		auto hash = Load<hash_t>(ptr);
+		auto hash = Load<hash_t>(ptr + hash_offset);
 
 		hashes_ptr[group_idx] = hash;
-		addresses_ptr[group_idx] = ptr + HASH_WIDTH;
+		addresses_ptr[group_idx] = ptr;
 		group_idx++;
 		if (group_idx == STANDARD_VECTOR_SIZE) {
 			FlushMove(addresses, hashes, group_idx);
@@ -819,7 +590,7 @@ void GroupedAggregateHashTable::Combine(GroupedAggregateHashTable &other) {
 		}
 	});
 	FlushMove(addresses, hashes, group_idx);
-	string_heap.MergeHeap(other.string_heap);
+	string_heap->Merge(*other.string_heap);
 	Verify();
 }
 
@@ -841,7 +612,7 @@ void GroupedAggregateHashTable::Partition(vector<GroupedAggregateHashTable *> &p
 	vector<PartitionInfo> partition_info(partition_hts.size());
 
 	PayloadApply([&](idx_t page_nr, idx_t page_offset, data_ptr_t ptr) {
-		auto hash = Load<hash_t>(ptr);
+		auto hash = Load<hash_t>(ptr + hash_offset);
 
 		idx_t partition = (hash & mask) >> shift;
 		D_ASSERT(partition < partition_hts.size());
@@ -849,7 +620,7 @@ void GroupedAggregateHashTable::Partition(vector<GroupedAggregateHashTable *> &p
 		auto &info = partition_info[partition];
 
 		info.hashes_ptr[info.group_count] = hash;
-		info.addresses_ptr[info.group_count] = ptr + HASH_WIDTH;
+		info.addresses_ptr[info.group_count] = ptr;
 		info.group_count++;
 		if (info.group_count == STANDARD_VECTOR_SIZE) {
 			D_ASSERT(partition_hts[partition]);
@@ -864,7 +635,7 @@ void GroupedAggregateHashTable::Partition(vector<GroupedAggregateHashTable *> &p
 		auto &info = partition_info[info_idx++];
 		partition_entry->FlushMove(info.addresses, info.hashes, info.group_count);
 
-		partition_entry->string_heap.MergeHeap(string_heap);
+		partition_entry->string_heap->Merge(*string_heap);
 		partition_entry->Verify();
 		total_count += partition_entry->Size();
 	}
@@ -886,7 +657,7 @@ idx_t GroupedAggregateHashTable::Scan(idx_t &scan_position, DataChunk &result) {
 
 	auto read_ptr = payload_hds_ptrs[chunk_idx++];
 	for (idx_t i = 0; i < this_n; i++) {
-		data_pointers[i] = read_ptr + chunk_offset + HASH_WIDTH;
+		data_pointers[i] = read_ptr + chunk_offset;
 		chunk_offset += tuple_size;
 		if (chunk_offset >= tuples_per_block * tuple_size) {
 			read_ptr = payload_hds_ptrs[chunk_idx++];
@@ -895,20 +666,16 @@ idx_t GroupedAggregateHashTable::Scan(idx_t &scan_position, DataChunk &result) {
 	}
 
 	result.SetCardinality(this_n);
-	// fetch the group columns
-	for (idx_t i = 0; i < group_types.size(); i++) {
+	// fetch the group columns (ignoring the final hash column
+	const auto group_cols = layout.ColumnCount() - 1;
+	for (idx_t i = 0; i < group_cols; i++) {
 		auto &column = result.data[i];
-		VectorOperations::Gather::Set(addresses, column, result.size());
+		RowOperations::Gather(layout, addresses, FlatVector::INCREMENTAL_SELECTION_VECTOR, column,
+		                      FlatVector::INCREMENTAL_SELECTION_VECTOR, result.size(), i);
 	}
 
-	VectorOperations::AddInPlace(addresses, group_padding, result.size());
+	RowOperations::FinalizeStates(layout, addresses, result, group_cols);
 
-	for (idx_t i = 0; i < aggregates.size(); i++) {
-		auto &target = result.data[group_types.size() + i];
-		auto &aggr = aggregates[i];
-		aggr.function.finalize(addresses, aggr.bind_data, target, result.size());
-		VectorOperations::AddInPlace(addresses, aggr.payload_size, result.size());
-	}
 	scan_position += this_n;
 	return this_n;
 }
