@@ -8,8 +8,9 @@
 
 namespace duckdb {
 
-ART::ART(vector<column_t> column_ids, vector<unique_ptr<Expression>> unbound_expressions, bool is_unique)
-    : Index(IndexType::ART, move(column_ids), move(unbound_expressions)), is_unique(is_unique) {
+ART::ART(const vector<column_t> &column_ids, const vector<unique_ptr<Expression>> &unbound_expressions, bool is_unique,
+         bool is_primary)
+    : Index(IndexType::ART, column_ids, unbound_expressions, is_unique, is_primary) {
 	tree = nullptr;
 	expression_result.Initialize(logical_types);
 	is_little_endian = IsLittleEndian();
