@@ -270,7 +270,7 @@ idx_t NestedLoopJoinComparisonSwitch(Vector &left, Vector &right, idx_t left_siz
 
 idx_t NestedLoopJoinInner::Perform(idx_t &lpos, idx_t &rpos, DataChunk &left_conditions, DataChunk &right_conditions,
                                    SelectionVector &lvector, SelectionVector &rvector,
-                                   vector<JoinCondition> &conditions) {
+                                   const vector<JoinCondition> &conditions) {
 	D_ASSERT(left_conditions.ColumnCount() == right_conditions.ColumnCount());
 	if (lpos >= left_conditions.size() || rpos >= right_conditions.size()) {
 		return 0;
