@@ -86,7 +86,7 @@ static void MarkJoinComparisonSwitch(Vector &left, Vector &right, idx_t lcount,
 }
 
 void NestedLoopJoinMark::Perform(DataChunk &left, ChunkCollection &right, bool found_match[],
-                                 vector<JoinCondition> &conditions) {
+                                 const vector<JoinCondition> &conditions) {
 	// initialize a new temporary selection vector for the left chunk
 	// loop over all chunks in the RHS
 	for (idx_t chunk_idx = 0; chunk_idx < right.ChunkCount(); chunk_idx++) {
