@@ -27,7 +27,7 @@ unique_ptr<GlobalOperatorState> PhysicalCrossProduct::GetGlobalState(ClientConte
 }
 
 void PhysicalCrossProduct::Sink(ExecutionContext &context, GlobalOperatorState &state, LocalSinkState &lstate_p,
-                                DataChunk &input) {
+                                DataChunk &input) const {
 	auto &sink = (CrossProductGlobalState &)state;
 	lock_guard<mutex> client_guard(sink.rhs_lock);
 	sink.rhs_materialized.Append(input);
@@ -52,7 +52,7 @@ unique_ptr<PhysicalOperatorState> PhysicalCrossProduct::GetOperatorState() {
 }
 
 void PhysicalCrossProduct::GetChunkInternal(ExecutionContext &context, DataChunk &chunk,
-                                            PhysicalOperatorState *state_p) {
+                                            PhysicalOperatorState *state_p) const {
 	auto state = reinterpret_cast<PhysicalCrossProductOperatorState *>(state_p);
 	auto &sink = (CrossProductGlobalState &)*sink_state;
 	auto &right_collection = sink.rhs_materialized;
