@@ -12,7 +12,7 @@ unique_ptr<PhysicalOperator> PhysicalPlanGenerator::CreatePlan(LogicalUpdate &op
 
 	dependencies.insert(op.table);
 	auto update = make_unique<PhysicalUpdate>(op.types, *op.table, *op.table->storage, op.columns, move(op.expressions),
-	                                          move(op.bound_defaults));
+	                                          move(op.bound_defaults), op.estimated_cardinality);
 	update->is_index_update = op.is_index_update;
 	update->children.push_back(move(plan));
 	return move(update);
