@@ -6,29 +6,31 @@
 
 namespace duckdb {
 
-struct string_agg_state_t {
+struct StringAggState {
 	idx_t size;
 	idx_t alloc_size;
 	char *dataptr;
 };
 
 struct StringAggBaseFunction {
-	template <class STATE> static void Initialize(STATE *state) {
+	template <class STATE>
+	static void Initialize(STATE *state) {
 		state->dataptr = nullptr;
 		state->alloc_size = 0;
 		state->size = 0;
 	}
 
 	template <class T, class STATE>
-	static void Finalize(Vector &result, FunctionData *, STATE *state, T *target, nullmask_t &nullmask, idx_t idx) {
+	static void Finalize(Vector &result, FunctionData *, STATE *state, T *target, ValidityMask &mask, idx_t idx) {
 		if (!state->dataptr) {
-			nullmask[idx] = true;
+			mask.SetInvalid(idx);
 		} else {
 			target[idx] = StringVector::AddString(result, state->dataptr, state->size);
 		}
 	}
 
-	template <class STATE> static void Destroy(STATE *state) {
+	template <class STATE>
+	static void Destroy(STATE *state) {
 		if (state->dataptr) {
 			delete[] state->dataptr;
 		}
@@ -38,7 +40,7 @@ struct StringAggBaseFunction {
 		return true;
 	}
 
-	static inline void PerformOperation(string_agg_state_t *state, const char *str, const char *sep, idx_t str_size,
+	static inline void PerformOperation(StringAggState *state, const char *str, const char *sep, idx_t str_size,
 	                                    idx_t sep_size) {
 		if (state->dataptr == nullptr) {
 			// first iteration: allocate space for the string and copy it into the state
@@ -68,37 +70,40 @@ struct StringAggBaseFunction {
 		}
 	}
 
-	static inline void PerformOperation(string_agg_state_t *state, string_t str, string_t sep) {
+	static inline void PerformOperation(StringAggState *state, string_t str, string_t sep) {
 		PerformOperation(state, str.GetDataUnsafe(), sep.GetDataUnsafe(), str.GetSize(), sep.GetSize());
 	}
 
-	static inline void PerformOperation(string_agg_state_t *state, string_t str) {
+	static inline void PerformOperation(StringAggState *state, string_t str) {
 		PerformOperation(state, str.GetDataUnsafe(), ",", str.GetSize(), 1);
 	}
 };
 
 struct StringAggFunction : public StringAggBaseFunction {
 	template <class A_TYPE, class B_TYPE, class STATE, class OP>
-	static void Operation(STATE *state, A_TYPE *str_data, B_TYPE *sep_data, nullmask_t &str_nullmask,
-	                      nullmask_t &sep_nullmask, idx_t str_idx, idx_t sep_idx) {
+	static void Operation(STATE *state, FunctionData *bind_data, A_TYPE *str_data, B_TYPE *sep_data,
+	                      ValidityMask &str_mask, ValidityMask &sep_mask, idx_t str_idx, idx_t sep_idx) {
 		PerformOperation(state, str_data[str_idx], sep_data[sep_idx]);
 	}
 };
 
 struct StringAggSingleFunction : public StringAggBaseFunction {
 	template <class INPUT_TYPE, class STATE, class OP>
-	static void Operation(STATE *state, INPUT_TYPE *str_data, nullmask_t &str_nullmask, idx_t str_idx) {
+	static void Operation(STATE *state, FunctionData *bind_data, INPUT_TYPE *str_data, ValidityMask &str_mask,
+	                      idx_t str_idx) {
 		PerformOperation(state, str_data[str_idx]);
 	}
 
 	template <class INPUT_TYPE, class STATE, class OP>
-	static void ConstantOperation(STATE *state, INPUT_TYPE *input, nullmask_t &nullmask, idx_t count) {
+	static void ConstantOperation(STATE *state, FunctionData *bind_data, INPUT_TYPE *input, ValidityMask &mask,
+	                              idx_t count) {
 		for (idx_t i = 0; i < count; i++) {
-			Operation<INPUT_TYPE, STATE, OP>(state, input, nullmask, 0);
+			Operation<INPUT_TYPE, STATE, OP>(state, bind_data, input, mask, 0);
 		}
 	}
 
-	template <class STATE, class OP> static void Combine(STATE source, STATE *target) {
+	template <class STATE, class OP>
+	static void Combine(const STATE &source, STATE *target) {
 		if (source.dataptr == nullptr) {
 			// source is not set: skip combining
 			return;
@@ -109,22 +114,22 @@ struct StringAggSingleFunction : public StringAggBaseFunction {
 
 void StringAggFun::RegisterFunction(BuiltinFunctions &set) {
 	AggregateFunctionSet string_agg("string_agg");
-	string_agg.AddFunction(AggregateFunction(
-	    {LogicalType::VARCHAR, LogicalType::VARCHAR}, LogicalType::VARCHAR,
-	    AggregateFunction::StateSize<string_agg_state_t>,
-	    AggregateFunction::StateInitialize<string_agg_state_t, StringAggFunction>,
-	    AggregateFunction::BinaryScatterUpdate<string_agg_state_t, string_t, string_t, StringAggFunction>, nullptr,
-	    AggregateFunction::StateFinalize<string_agg_state_t, string_t, StringAggFunction>,
-	    AggregateFunction::BinaryUpdate<string_agg_state_t, string_t, string_t, StringAggFunction>, nullptr,
-	    AggregateFunction::StateDestroy<string_agg_state_t, StringAggFunction>));
-	string_agg.AddFunction(AggregateFunction(
-	    {LogicalType::VARCHAR}, LogicalType::VARCHAR, AggregateFunction::StateSize<string_agg_state_t>,
-	    AggregateFunction::StateInitialize<string_agg_state_t, StringAggSingleFunction>,
-	    AggregateFunction::UnaryScatterUpdate<string_agg_state_t, string_t, StringAggSingleFunction>,
-	    AggregateFunction::StateCombine<string_agg_state_t, StringAggSingleFunction>,
-	    AggregateFunction::StateFinalize<string_agg_state_t, string_t, StringAggSingleFunction>,
-	    AggregateFunction::UnaryUpdate<string_agg_state_t, string_t, StringAggSingleFunction>, nullptr,
-	    AggregateFunction::StateDestroy<string_agg_state_t, StringAggSingleFunction>));
+	string_agg.AddFunction(
+	    AggregateFunction({LogicalType::VARCHAR, LogicalType::VARCHAR}, LogicalType::VARCHAR,
+	                      AggregateFunction::StateSize<StringAggState>,
+	                      AggregateFunction::StateInitialize<StringAggState, StringAggFunction>,
+	                      AggregateFunction::BinaryScatterUpdate<StringAggState, string_t, string_t, StringAggFunction>,
+	                      nullptr, AggregateFunction::StateFinalize<StringAggState, string_t, StringAggFunction>,
+	                      AggregateFunction::BinaryUpdate<StringAggState, string_t, string_t, StringAggFunction>,
+	                      nullptr, AggregateFunction::StateDestroy<StringAggState, StringAggFunction>));
+	string_agg.AddFunction(
+	    AggregateFunction({LogicalType::VARCHAR}, LogicalType::VARCHAR, AggregateFunction::StateSize<StringAggState>,
+	                      AggregateFunction::StateInitialize<StringAggState, StringAggSingleFunction>,
+	                      AggregateFunction::UnaryScatterUpdate<StringAggState, string_t, StringAggSingleFunction>,
+	                      AggregateFunction::StateCombine<StringAggState, StringAggSingleFunction>,
+	                      AggregateFunction::StateFinalize<StringAggState, string_t, StringAggSingleFunction>,
+	                      AggregateFunction::UnaryUpdate<StringAggState, string_t, StringAggSingleFunction>, nullptr,
+	                      AggregateFunction::StateDestroy<StringAggState, StringAggSingleFunction>));
 	set.AddFunction(string_agg);
 	string_agg.name = "group_concat";
 	set.AddFunction(string_agg);
