@@ -1,15 +1,15 @@
+#include "duckdb/function/scalar/nested_functions.hpp"
 #include "duckdb/function/aggregate/nested_functions.hpp"
 #include "duckdb/planner/expression/bound_aggregate_expression.hpp"
 #include "duckdb/common/pair.hpp"
 #include "duckdb/planner/expression/bound_function_expression.hpp"
 #include "duckdb/common/map.hpp"
 #include "duckdb/common/types/vector.hpp"
-#include "duckdb/common/types/chunk_collection.hpp"
 
 namespace duckdb {
 template <class T>
 struct HistogramAggState {
-	map<T, size_t> *hist;
+	map<T, idx_t> *hist;
 };
 
 struct HistogramFunction {
@@ -46,7 +46,7 @@ static void HistogramUpdateFunction(Vector inputs[], FunctionData *, idx_t input
 		if (input_data.validity.RowIsValid(input_data.sel->get_index(i))) {
 			auto state = states[sdata.sel->get_index(i)];
 			if (!state->hist) {
-				state->hist = new map<T, size_t>();
+				state->hist = new map<T, idx_t>();
 			}
 			auto value = (T *)input_data.data;
 			(*state->hist)[value[input_data.sel->get_index(i)]]++;
@@ -69,7 +69,7 @@ static void HistogramUpdateFunctionString(Vector inputs[], FunctionData *, idx_t
 		if (input_data.validity.RowIsValid(input_data.sel->get_index(i))) {
 			auto state = states[sdata.sel->get_index(i)];
 			if (!state->hist) {
-				state->hist = new map<string, size_t>();
+				state->hist = new map<string, idx_t>();
 			}
 			auto value = (string_t *)input_data.data;
 			(*state->hist)[value[input_data.sel->get_index(i)].GetString()]++;
@@ -91,7 +91,7 @@ static void HistogramCombineFunction(Vector &state, Vector &combined, idx_t coun
 			continue;
 		}
 		if (!combined_ptr[i]->hist) {
-			combined_ptr[i]->hist = new map<T, size_t>();
+			combined_ptr[i]->hist = new map<T, idx_t>();
 		}
 		D_ASSERT(combined_ptr[i]->hist);
 		D_ASSERT(state->hist);
@@ -106,26 +106,38 @@ static void HistogramFinalize(Vector &state_vector, FunctionData *, Vector &resu
 	VectorData sdata;
 	state_vector.Orrify(count, sdata);
 	auto states = (HistogramAggState<T> **)sdata.data;
-	result.Initialize(result.GetType());
-	auto list_struct_data = FlatVector::GetData<list_entry_t>(result);
-	auto list_child = make_unique<Vector>(result.GetType().child_types()[0].second);
-	size_t old_len = 0;
-	ListVector::SetEntry(result, move(list_child));
+	result.Initialize();
+
+	idx_t old_len = 0;
+
 	auto &mask = FlatVector::Validity(result);
+
+	auto &child_entries = StructVector::GetEntries(result);
+	auto &bucket_list = child_entries[0];
+	auto &count_list = child_entries[1];
+
+	auto &bucket_validity = FlatVector::Validity(*bucket_list);
+	auto &count_validity = FlatVector::Validity(*count_list);
 	for (idx_t i = 0; i < count; i++) {
 		auto state = states[sdata.sel->get_index(i)];
 		if (!state->hist) {
 			mask.SetInvalid(i);
+			bucket_validity.SetInvalid(i);
+			count_validity.SetInvalid(i);
 			continue;
 		}
 		for (auto &entry : *state->hist) {
-			child_list_t<Value> struct_values;
-			struct_values.push_back({"bucket", Value::CreateValue(entry.first)});
-			struct_values.push_back({"count", Value::UBIGINT(entry.second)});
-			auto val = Value::STRUCT(struct_values);
-			ListVector::PushBack(result, val);
+			auto bucket_value = Value::CreateValue(entry.first);
+			ListVector::PushBack(*bucket_list, bucket_value);
+			auto count_value = Value::CreateValue(entry.second);
+			ListVector::PushBack(*count_list, count_value);
 		}
-		list_struct_data[i].length = ListVector::GetListSize(result) - old_len;
+		auto list_struct_data = FlatVector::GetData<list_entry_t>(*bucket_list);
+		list_struct_data[i].length = ListVector::GetListSize(*bucket_list) - old_len;
+		list_struct_data[i].offset = old_len;
+
+		list_struct_data = FlatVector::GetData<list_entry_t>(*count_list);
+		list_struct_data[i].length = ListVector::GetListSize(*count_list) - old_len;
 		list_struct_data[i].offset = old_len;
 		old_len = list_struct_data[i].length;
 	}
@@ -138,77 +150,74 @@ unique_ptr<FunctionData> HistogramBindFunction(ClientContext &context, Aggregate
 	}
 	D_ASSERT(arguments.size() == 1);
 	child_list_t<LogicalType> struct_children;
-	struct_children.push_back({"bucket", arguments[0]->return_type});
-	struct_children.push_back({"count", LogicalType::UBIGINT});
-	auto struct_type = LogicalType(LogicalTypeId::STRUCT, move(struct_children));
-	child_list_t<LogicalType> children;
-	children.push_back(make_pair("", struct_type));
+	struct_children.push_back({"bucket", LogicalType::LIST(arguments[0]->return_type)});
+	struct_children.push_back({"count", LogicalType::LIST(LogicalType::UBIGINT)});
+	auto struct_type = LogicalType::MAP(move(struct_children));
 
-	function.return_type = LogicalType(LogicalTypeId::LIST, move(children));
-	return make_unique<ListBindData>(); // TODO atm this is not used anywhere but it might not be required after all
-	                                    // except for sanity checking
+	function.return_type = struct_type;
+	return make_unique<VariableReturnBindData>(function.return_type);
 }
 
 AggregateFunction GetHistogramFunction(PhysicalType type) {
 	switch (type) {
 	case PhysicalType::UINT16:
-		return AggregateFunction("histogram", {LogicalType::USMALLINT}, LogicalType::STRUCT,
+		return AggregateFunction("histogram", {LogicalType::USMALLINT}, LogicalTypeId::MAP,
 		                         AggregateFunction::StateSize<HistogramAggState<uint16_t>>,
 		                         AggregateFunction::StateInitialize<HistogramAggState<uint16_t>, HistogramFunction>,
 		                         HistogramUpdateFunction<uint16_t>, HistogramCombineFunction<uint16_t>,
 		                         HistogramFinalize<uint16_t>, nullptr, HistogramBindFunction,
 		                         AggregateFunction::StateDestroy<HistogramAggState<uint16_t>, HistogramFunction>);
 	case PhysicalType::UINT32:
-		return AggregateFunction("histogram", {LogicalType::UINTEGER}, LogicalType::STRUCT,
+		return AggregateFunction("histogram", {LogicalType::UINTEGER}, LogicalTypeId::MAP,
 		                         AggregateFunction::StateSize<HistogramAggState<uint32_t>>,
 		                         AggregateFunction::StateInitialize<HistogramAggState<uint32_t>, HistogramFunction>,
 		                         HistogramUpdateFunction<uint32_t>, HistogramCombineFunction<uint32_t>,
 		                         HistogramFinalize<uint32_t>, nullptr, HistogramBindFunction,
 		                         AggregateFunction::StateDestroy<HistogramAggState<uint32_t>, HistogramFunction>);
 	case PhysicalType::UINT64:
-		return AggregateFunction("histogram", {LogicalType::UBIGINT}, LogicalType::STRUCT,
+		return AggregateFunction("histogram", {LogicalType::UBIGINT}, LogicalTypeId::MAP,
 		                         AggregateFunction::StateSize<HistogramAggState<uint64_t>>,
 		                         AggregateFunction::StateInitialize<HistogramAggState<uint64_t>, HistogramFunction>,
 		                         HistogramUpdateFunction<uint64_t>, HistogramCombineFunction<uint64_t>,
 		                         HistogramFinalize<uint64_t>, nullptr, HistogramBindFunction,
 		                         AggregateFunction::StateDestroy<HistogramAggState<uint64_t>, HistogramFunction>);
 	case PhysicalType::INT16:
-		return AggregateFunction("histogram", {LogicalType::SMALLINT}, LogicalType::STRUCT,
+		return AggregateFunction("histogram", {LogicalType::SMALLINT}, LogicalTypeId::MAP,
 		                         AggregateFunction::StateSize<HistogramAggState<int16_t>>,
 		                         AggregateFunction::StateInitialize<HistogramAggState<int16_t>, HistogramFunction>,
 		                         HistogramUpdateFunction<int16_t>, HistogramCombineFunction<int16_t>,
 		                         HistogramFinalize<int16_t>, nullptr, HistogramBindFunction,
 		                         AggregateFunction::StateDestroy<HistogramAggState<int16_t>, HistogramFunction>);
 	case PhysicalType::INT32:
-		return AggregateFunction("histogram", {LogicalType::INTEGER}, LogicalType::STRUCT,
+		return AggregateFunction("histogram", {LogicalType::INTEGER}, LogicalTypeId::MAP,
 		                         AggregateFunction::StateSize<HistogramAggState<int32_t>>,
 		                         AggregateFunction::StateInitialize<HistogramAggState<int32_t>, HistogramFunction>,
 		                         HistogramUpdateFunction<int32_t>, HistogramCombineFunction<int32_t>,
 		                         HistogramFinalize<int32_t>, nullptr, HistogramBindFunction,
 		                         AggregateFunction::StateDestroy<HistogramAggState<int32_t>, HistogramFunction>);
 	case PhysicalType::INT64:
-		return AggregateFunction("histogram", {LogicalType::BIGINT}, LogicalType::STRUCT,
+		return AggregateFunction("histogram", {LogicalType::BIGINT}, LogicalTypeId::MAP,
 		                         AggregateFunction::StateSize<HistogramAggState<int64_t>>,
 		                         AggregateFunction::StateInitialize<HistogramAggState<int64_t>, HistogramFunction>,
 		                         HistogramUpdateFunction<int64_t>, HistogramCombineFunction<int64_t>,
 		                         HistogramFinalize<int64_t>, nullptr, HistogramBindFunction,
 		                         AggregateFunction::StateDestroy<HistogramAggState<int64_t>, HistogramFunction>);
 	case PhysicalType::FLOAT:
-		return AggregateFunction("histogram", {LogicalType::FLOAT}, LogicalType::STRUCT,
+		return AggregateFunction("histogram", {LogicalType::FLOAT}, LogicalTypeId::MAP,
 		                         AggregateFunction::StateSize<HistogramAggState<float>>,
 		                         AggregateFunction::StateInitialize<HistogramAggState<float>, HistogramFunction>,
 		                         HistogramUpdateFunction<float>, HistogramCombineFunction<float>,
 		                         HistogramFinalize<float>, nullptr, HistogramBindFunction,
 		                         AggregateFunction::StateDestroy<HistogramAggState<float>, HistogramFunction>);
 	case PhysicalType::DOUBLE:
-		return AggregateFunction("histogram", {LogicalType::DOUBLE}, LogicalType::STRUCT,
+		return AggregateFunction("histogram", {LogicalType::DOUBLE}, LogicalTypeId::MAP,
 		                         AggregateFunction::StateSize<HistogramAggState<double>>,
 		                         AggregateFunction::StateInitialize<HistogramAggState<double>, HistogramFunction>,
 		                         HistogramUpdateFunction<double>, HistogramCombineFunction<double>,
 		                         HistogramFinalize<double>, nullptr, HistogramBindFunction,
 		                         AggregateFunction::StateDestroy<HistogramAggState<double>, HistogramFunction>);
 	case PhysicalType::VARCHAR:
-		return AggregateFunction("histogram", {LogicalType::VARCHAR}, LogicalType::STRUCT,
+		return AggregateFunction("histogram", {LogicalType::VARCHAR}, LogicalTypeId::MAP,
 		                         AggregateFunction::StateSize<HistogramAggState<string>>,
 		                         AggregateFunction::StateInitialize<HistogramAggState<string>, HistogramFunction>,
 		                         HistogramUpdateFunctionString, HistogramCombineFunction<string>,
@@ -231,7 +240,7 @@ void HistogramFun::RegisterFunction(BuiltinFunctions &set) {
 	fun.AddFunction(GetHistogramFunction(PhysicalType::FLOAT));
 	fun.AddFunction(GetHistogramFunction(PhysicalType::DOUBLE));
 	fun.AddFunction(GetHistogramFunction(PhysicalType::VARCHAR));
-	fun.AddFunction(AggregateFunction("histogram", {LogicalType::TIMESTAMP}, LogicalType::STRUCT,
+	fun.AddFunction(AggregateFunction("histogram", {LogicalType::TIMESTAMP}, LogicalTypeId::MAP,
 	                                  AggregateFunction::StateSize<HistogramAggState<int64_t>>,
 	                                  AggregateFunction::StateInitialize<HistogramAggState<int64_t>, HistogramFunction>,
 	                                  HistogramUpdateFunction<int64_t>, HistogramCombineFunction<int64_t>,
