@@ -11,18 +11,16 @@ static void StructPackFunction(DataChunk &args, ExpressionState &state, Vector &
 	auto &info = (VariableReturnBindData &)*func_expr.bind_info;
 
 	// this should never happen if the binder below is sane
-	D_ASSERT(args.ColumnCount() == info.stype.child_types().size());
+	D_ASSERT(args.ColumnCount() == StructType::GetChildTypes(info.stype).size());
 
 	bool all_const = true;
+	auto &child_entries = StructVector::GetEntries(result);
 	for (size_t i = 0; i < args.ColumnCount(); i++) {
 		if (args.data[i].GetVectorType() != VectorType::CONSTANT_VECTOR) {
 			all_const = false;
 		}
 		// same holds for this
-		D_ASSERT(args.data[i].GetType() == info.stype.child_types()[i].second);
-		auto new_child = make_unique<Vector>(info.stype.child_types()[i].second);
-		new_child->Reference(args.data[i]);
-		StructVector::AddEntry(result, info.stype.child_types()[i].first, move(new_child));
+		child_entries[i]->Reference(args.data[i]);
 	}
 	result.SetVectorType(all_const ? VectorType::CONSTANT_VECTOR : VectorType::FLAT_VECTOR);
 
@@ -41,26 +39,26 @@ static unique_ptr<FunctionData> StructPackBind(ClientContext &context, ScalarFun
 	for (idx_t i = 0; i < arguments.size(); i++) {
 		auto &child = arguments[i];
 		if (child->alias.empty() && bound_function.name == "struct_pack") {
-			throw Exception("Need named argument for struct pack, e.g. STRUCT_PACK(a := b)");
+			throw BinderException("Need named argument for struct pack, e.g. STRUCT_PACK(a := b)");
 		}
 		if (child->alias.empty() && bound_function.name == "row") {
 			child->alias = "v" + std::to_string(i + 1);
 		}
 		if (name_collision_set.find(child->alias) != name_collision_set.end()) {
-			throw Exception("Duplicate struct entry name");
+			throw BinderException("Duplicate struct entry name \"%s\"", child->alias);
 		}
 		name_collision_set.insert(child->alias);
 		struct_children.push_back(make_pair(child->alias, arguments[i]->return_type));
 	}
 
 	// this is more for completeness reasons
-	bound_function.return_type = LogicalType(LogicalTypeId::STRUCT, move(struct_children));
+	bound_function.return_type = LogicalType::STRUCT(move(struct_children));
 	return make_unique<VariableReturnBindData>(bound_function.return_type);
 }
 
 void StructPackFun::RegisterFunction(BuiltinFunctions &set) {
 	// the arguments and return types are actually set in the binder function
-	ScalarFunction fun("struct_pack", {}, LogicalType::STRUCT, StructPackFunction, false, StructPackBind);
+	ScalarFunction fun("struct_pack", {}, LogicalTypeId::STRUCT, StructPackFunction, false, StructPackBind);
 	fun.varargs = LogicalType::ANY;
 	set.AddFunction(fun);
 	fun.name = "row";
