@@ -15,12 +15,12 @@ struct StandardCharacterReader {
 
 struct ASCIILCaseReader {
 	static char Operation(const char *data, idx_t pos) {
-		return (char)LowerFun::ASCIIToLowerMap[(uint8_t)data[pos]];
+		return (char)LowerFun::ascii_to_lower_map[(uint8_t)data[pos]];
 	}
 };
 
 template <char PERCENTAGE, char UNDERSCORE, class READER = StandardCharacterReader>
-bool templated_like_operator(const char *sdata, idx_t slen, const char *pdata, idx_t plen, char escape) {
+bool TemplatedLikeOperator(const char *sdata, idx_t slen, const char *pdata, idx_t plen, char escape) {
 	idx_t pidx = 0;
 	idx_t sidx = 0;
 	for (; pidx < plen && sidx < slen; pidx++) {
@@ -46,8 +46,8 @@ bool templated_like_operator(const char *sdata, idx_t slen, const char *pdata, i
 				return true; /* tail is acceptable */
 			}
 			for (; sidx < slen; sidx++) {
-				if (templated_like_operator<PERCENTAGE, UNDERSCORE, READER>(sdata + sidx, slen - sidx, pdata + pidx,
-				                                                            plen - pidx, escape)) {
+				if (TemplatedLikeOperator<PERCENTAGE, UNDERSCORE, READER>(sdata + sidx, slen - sidx, pdata + pidx,
+				                                                          plen - pidx, escape)) {
 					return true;
 				}
 			}
@@ -65,7 +65,7 @@ bool templated_like_operator(const char *sdata, idx_t slen, const char *pdata, i
 }
 
 struct LikeSegment {
-	LikeSegment(string pattern) : pattern(move(pattern)) {
+	explicit LikeSegment(string pattern) : pattern(move(pattern)) {
 	}
 
 	string pattern;
@@ -82,7 +82,7 @@ struct LikeMatcher : public FunctionData {
 		idx_t segment_idx = 0;
 		idx_t end_idx = segments.size() - 1;
 		if (!has_start_percentage) {
-			// no start percentage: match the first part of the string directly
+			// no start sample_size: match the first part of the string directly
 			auto &segment = segments[0];
 			if (str_len < segment.pattern.size()) {
 				return false;
@@ -95,7 +95,7 @@ struct LikeMatcher : public FunctionData {
 			segment_idx++;
 			if (segments.size() == 1) {
 				// only one segment, and it matches
-				// we have a match if there is an end percentage, OR if the memcmp was an exact match (remaining str is
+				// we have a match if there is an end sample_size, OR if the memcmp was an exact match (remaining str is
 				// empty)
 				return has_end_percentage || str_len == 0;
 			}
@@ -116,7 +116,7 @@ struct LikeMatcher : public FunctionData {
 		}
 		if (!has_end_percentage) {
 			end_idx--;
-			// no end percentage: match the final segment now
+			// no end sample_size: match the final segment now
 			auto &segment = segments.back();
 			if (str_len < segment.pattern.size()) {
 				return false;
@@ -145,7 +145,7 @@ struct LikeMatcher : public FunctionData {
 			if (ch == escape || ch == '%' || ch == '_') {
 				// special character, push a constant pattern
 				if (i > last_non_pattern) {
-					segments.push_back(LikeSegment(like_pattern.substr(last_non_pattern, i - last_non_pattern)));
+					segments.emplace_back(like_pattern.substr(last_non_pattern, i - last_non_pattern));
 				}
 				last_non_pattern = i + 1;
 				if (ch == escape || ch == '_') {
@@ -153,7 +153,7 @@ struct LikeMatcher : public FunctionData {
 					// FIXME: we could handle escaped percentages here
 					return nullptr;
 				} else {
-					// percentage
+					// sample_size
 					if (i == 0) {
 						has_start_percentage = true;
 					}
@@ -164,8 +164,7 @@ struct LikeMatcher : public FunctionData {
 			}
 		}
 		if (last_non_pattern < like_pattern.size()) {
-			segments.push_back(
-			    LikeSegment(like_pattern.substr(last_non_pattern, like_pattern.size() - last_non_pattern)));
+			segments.emplace_back(like_pattern.substr(last_non_pattern, like_pattern.size() - last_non_pattern));
 		}
 		if (segments.empty()) {
 			return nullptr;
@@ -183,8 +182,8 @@ private:
 	bool has_end_percentage;
 };
 
-static unique_ptr<FunctionData> like_bind_function(ClientContext &context, ScalarFunction &bound_function,
-                                                   vector<unique_ptr<Expression>> &arguments) {
+static unique_ptr<FunctionData> LikeBindFunction(ClientContext &context, ScalarFunction &bound_function,
+                                                 vector<unique_ptr<Expression>> &arguments) {
 	// pattern is the second argument. If its constant, we can already prepare the pattern and store it for later.
 	D_ASSERT(arguments.size() == 2 || arguments.size() == 3);
 	if (arguments[1]->IsFoldable()) {
@@ -194,12 +193,12 @@ static unique_ptr<FunctionData> like_bind_function(ClientContext &context, Scala
 	return nullptr;
 }
 
-bool like_operator(const char *s, idx_t slen, const char *pattern, idx_t plen, char escape) {
-	return templated_like_operator<'%', '_'>(s, slen, pattern, plen, escape);
+bool LikeOperatorFunction(const char *s, idx_t slen, const char *pattern, idx_t plen, char escape) {
+	return TemplatedLikeOperator<'%', '_'>(s, slen, pattern, plen, escape);
 }
 
-bool like_operator(string_t &s, string_t &pat, char escape = '\0') {
-	return like_operator(s.GetDataUnsafe(), s.GetSize(), pat.GetDataUnsafe(), pat.GetSize(), escape);
+bool LikeOperatorFunction(string_t &s, string_t &pat, char escape = '\0') {
+	return LikeOperatorFunction(s.GetDataUnsafe(), s.GetSize(), pat.GetDataUnsafe(), pat.GetSize(), escape);
 }
 
 bool LikeFun::Glob(const char *string, idx_t slen, const char *pattern, idx_t plen) {
@@ -337,31 +336,35 @@ parse_bracket : {
 }
 
 struct LikeEscapeOperator {
-	template <class TA, class TB, class TC> static inline bool Operation(TA str, TB pattern, TC escape) {
+	template <class TA, class TB, class TC>
+	static inline bool Operation(TA str, TB pattern, TC escape) {
 		// Only one escape character should be allowed
 		if (escape.GetSize() > 1) {
 			throw SyntaxException("Invalid escape string. Escape string must be empty or one character.");
 		}
 		char escape_char = escape.GetSize() == 0 ? '\0' : *escape.GetDataUnsafe();
-		return like_operator(str.GetDataUnsafe(), str.GetSize(), pattern.GetDataUnsafe(), pattern.GetSize(),
-		                     escape_char);
+		return LikeOperatorFunction(str.GetDataUnsafe(), str.GetSize(), pattern.GetDataUnsafe(), pattern.GetSize(),
+		                            escape_char);
 	}
 };
 
 struct NotLikeEscapeOperator {
-	template <class TA, class TB, class TC> static inline bool Operation(TA str, TB pattern, TC escape) {
+	template <class TA, class TB, class TC>
+	static inline bool Operation(TA str, TB pattern, TC escape) {
 		return !LikeEscapeOperator::Operation(str, pattern, escape);
 	}
 };
 
 struct LikeOperator {
-	template <class TA, class TB, class TR> static inline TR Operation(TA str, TB pattern) {
-		return like_operator(str, pattern);
+	template <class TA, class TB, class TR>
+	static inline TR Operation(TA str, TB pattern) {
+		return LikeOperatorFunction(str, pattern);
 	}
 };
 
 struct ILikeOperator {
-	template <class TA, class TB, class TR> static inline TR Operation(TA str, TB pattern) {
+	template <class TA, class TB, class TR>
+	static inline TR Operation(TA str, TB pattern) {
 		auto str_data = str.GetDataUnsafe();
 		auto str_size = str.GetSize();
 		auto pat_data = pattern.GetDataUnsafe();
@@ -376,55 +379,61 @@ struct ILikeOperator {
 		LowerFun::LowerCase(pat_data, pat_size, pat_ldata.get());
 		string_t str_lcase(str_ldata.get(), str_llength);
 		string_t pat_lcase(pat_ldata.get(), pat_llength);
-		return like_operator(str_lcase, pat_lcase);
+		return LikeOperatorFunction(str_lcase, pat_lcase);
 	}
 };
 
 struct NotLikeOperator {
-	template <class TA, class TB, class TR> static inline TR Operation(TA str, TB pattern) {
-		return !like_operator(str, pattern);
+	template <class TA, class TB, class TR>
+	static inline TR Operation(TA str, TB pattern) {
+		return !LikeOperatorFunction(str, pattern);
 	}
 };
 
 struct NotILikeOperator {
-	template <class TA, class TB, class TR> static inline TR Operation(TA str, TB pattern) {
+	template <class TA, class TB, class TR>
+	static inline TR Operation(TA str, TB pattern) {
 		return !ILikeOperator::Operation<TA, TB, TR>(str, pattern);
 	}
 };
 
 struct ILikeOperatorASCII {
-	template <class TA, class TB, class TR> static inline TR Operation(TA str, TB pattern) {
-		return templated_like_operator<'%', '_', ASCIILCaseReader>(str.GetDataUnsafe(), str.GetSize(),
-		                                                           pattern.GetDataUnsafe(), pattern.GetSize(), '\0');
+	template <class TA, class TB, class TR>
+	static inline TR Operation(TA str, TB pattern) {
+		return TemplatedLikeOperator<'%', '_', ASCIILCaseReader>(str.GetDataUnsafe(), str.GetSize(),
+		                                                         pattern.GetDataUnsafe(), pattern.GetSize(), '\0');
 	}
 };
 
 struct NotILikeOperatorASCII {
-	template <class TA, class TB, class TR> static inline TR Operation(TA str, TB pattern) {
+	template <class TA, class TB, class TR>
+	static inline TR Operation(TA str, TB pattern) {
 		return !ILikeOperatorASCII::Operation<TA, TB, TR>(str, pattern);
 	}
 };
 
 struct GlobOperator {
-	template <class TA, class TB, class TR> static inline TR Operation(TA str, TB pattern) {
+	template <class TA, class TB, class TR>
+	static inline TR Operation(TA str, TB pattern) {
 		return LikeFun::Glob(str.GetDataUnsafe(), str.GetSize(), pattern.GetDataUnsafe(), pattern.GetSize());
 	}
 };
 
 // This can be moved to the scalar_function class
-template <typename Func> static void like_escape_function(DataChunk &args, ExpressionState &state, Vector &result) {
+template <typename FUNC>
+static void LikeEscapeFunction(DataChunk &args, ExpressionState &state, Vector &result) {
 	auto &str = args.data[0];
 	auto &pattern = args.data[1];
 	auto &escape = args.data[2];
 
 	TernaryExecutor::Execute<string_t, string_t, string_t, bool>(
-	    str, pattern, escape, result, args.size(), Func::template Operation<string_t, string_t, string_t>);
+	    str, pattern, escape, result, args.size(), FUNC::template Operation<string_t, string_t, string_t>);
 }
 
 template <class ASCII_OP>
-static unique_ptr<BaseStatistics> ilike_propagate_stats(ClientContext &context, BoundFunctionExpression &expr,
-                                                        FunctionData *bind_data,
-                                                        vector<unique_ptr<BaseStatistics>> &child_stats) {
+static unique_ptr<BaseStatistics> ILikePropagateStats(ClientContext &context, BoundFunctionExpression &expr,
+                                                      FunctionData *bind_data,
+                                                      vector<unique_ptr<BaseStatistics>> &child_stats) {
 	D_ASSERT(child_stats.size() >= 1);
 	// can only propagate stats if the children have stats
 	if (!child_stats[0]) {
@@ -432,7 +441,7 @@ static unique_ptr<BaseStatistics> ilike_propagate_stats(ClientContext &context,
 	}
 	auto &sstats = (StringStatistics &)*child_stats[0];
 	if (!sstats.has_unicode) {
-		expr.function.function = ScalarFunction::BinaryFunction<string_t, string_t, bool, ASCII_OP, true>;
+		expr.function.function = ScalarFunction::BinaryFunction<string_t, string_t, bool, ASCII_OP>;
 	}
 	return nullptr;
 }
@@ -443,40 +452,40 @@ static void RegularLikeFunction(DataChunk &input, ExpressionState &state, Vector
 	if (func_expr.bind_info) {
 		auto &matcher = (LikeMatcher &)*func_expr.bind_info;
 		// use fast like matcher
-		UnaryExecutor::Execute<string_t, bool, true>(input.data[0], result, input.size(), [&](string_t str) {
+		UnaryExecutor::Execute<string_t, bool>(input.data[0], result, input.size(), [&](string_t str) {
 			return INVERT ? !matcher.Match(str) : matcher.Match(str);
 		});
 	} else {
 		// use generic like matcher
-		BinaryExecutor::ExecuteStandard<string_t, string_t, bool, OP, true>(input.data[0], input.data[1], result,
-		                                                                    input.size());
+		BinaryExecutor::ExecuteStandard<string_t, string_t, bool, OP>(input.data[0], input.data[1], result,
+		                                                              input.size());
 	}
 }
 void LikeFun::RegisterFunction(BuiltinFunctions &set) {
 	// like
 	set.AddFunction(ScalarFunction("~~", {LogicalType::VARCHAR, LogicalType::VARCHAR}, LogicalType::BOOLEAN,
-	                               RegularLikeFunction<LikeOperator, false>, false, like_bind_function));
+	                               RegularLikeFunction<LikeOperator, false>, false, LikeBindFunction));
 	// not like
 	set.AddFunction(ScalarFunction("!~~", {LogicalType::VARCHAR, LogicalType::VARCHAR}, LogicalType::BOOLEAN,
-	                               RegularLikeFunction<NotLikeOperator, true>, false, like_bind_function));
+	                               RegularLikeFunction<NotLikeOperator, true>, false, LikeBindFunction));
 	// glob
 	set.AddFunction(ScalarFunction("~~~", {LogicalType::VARCHAR, LogicalType::VARCHAR}, LogicalType::BOOLEAN,
-	                               ScalarFunction::BinaryFunction<string_t, string_t, bool, GlobOperator, true>));
+	                               ScalarFunction::BinaryFunction<string_t, string_t, bool, GlobOperator>));
 	// ilike
 	set.AddFunction(ScalarFunction("~~*", {LogicalType::VARCHAR, LogicalType::VARCHAR}, LogicalType::BOOLEAN,
-	                               ScalarFunction::BinaryFunction<string_t, string_t, bool, ILikeOperator, true>, false,
-	                               nullptr, nullptr, ilike_propagate_stats<ILikeOperatorASCII>));
+	                               ScalarFunction::BinaryFunction<string_t, string_t, bool, ILikeOperator>, false,
+	                               nullptr, nullptr, ILikePropagateStats<ILikeOperatorASCII>));
 	// not ilike
 	set.AddFunction(ScalarFunction("!~~*", {LogicalType::VARCHAR, LogicalType::VARCHAR}, LogicalType::BOOLEAN,
-	                               ScalarFunction::BinaryFunction<string_t, string_t, bool, NotILikeOperator, true>,
-	                               false, nullptr, nullptr, ilike_propagate_stats<NotILikeOperatorASCII>));
+	                               ScalarFunction::BinaryFunction<string_t, string_t, bool, NotILikeOperator>, false,
+	                               nullptr, nullptr, ILikePropagateStats<NotILikeOperatorASCII>));
 }
 
 void LikeEscapeFun::RegisterFunction(BuiltinFunctions &set) {
 	set.AddFunction({"like_escape"}, ScalarFunction({LogicalType::VARCHAR, LogicalType::VARCHAR, LogicalType::VARCHAR},
-	                                                LogicalType::BOOLEAN, like_escape_function<LikeEscapeOperator>));
+	                                                LogicalType::BOOLEAN, LikeEscapeFunction<LikeEscapeOperator>));
 	set.AddFunction({"not_like_escape"},
 	                ScalarFunction({LogicalType::VARCHAR, LogicalType::VARCHAR, LogicalType::VARCHAR},
-	                               LogicalType::BOOLEAN, like_escape_function<NotLikeEscapeOperator>));
+	                               LogicalType::BOOLEAN, LikeEscapeFunction<NotLikeEscapeOperator>));
 }
 } // namespace duckdb
