@@ -1,10 +1,9 @@
 #include "duckdb/function/table/read_csv.hpp"
-
 #include "duckdb/execution/operator/persistent/buffered_csv_reader.hpp"
 #include "duckdb/function/function_set.hpp"
 #include "duckdb/main/client_context.hpp"
 #include "duckdb/main/database.hpp"
-
+#include "duckdb/common/string_util.hpp"
 #include "duckdb/main/config.hpp"
 #include "duckdb/parser/expression/constant_expression.hpp"
 #include "duckdb/parser/expression/function_expression.hpp"
@@ -94,15 +93,19 @@ static unique_ptr<FunctionData> ReadCSVBind(ClientContext &context, vector<Value
 				throw InvalidInputException("Could not parse TIMESTAMPFORMAT: %s", error.c_str());
 			}
 		} else if (kv.first == "columns") {
-			if (kv.second.type().id() != LogicalTypeId::STRUCT) {
+			auto &child_type = kv.second.type();
+			if (child_type.id() != LogicalTypeId::STRUCT) {
 				throw BinderException("read_csv columns requires a a struct as input");
 			}
-			for (auto &val : kv.second.struct_value) {
-				names.push_back(val.first);
-				if (val.second.type().id() != LogicalTypeId::VARCHAR) {
+			D_ASSERT(StructType::GetChildCount(child_type) == kv.second.struct_value.size());
+			for (idx_t i = 0; i < kv.second.struct_value.size(); i++) {
+				auto &name = StructType::GetChildName(child_type, i);
+				auto &val = kv.second.struct_value[i];
+				names.push_back(name);
+				if (val.type().id() != LogicalTypeId::VARCHAR) {
 					throw BinderException("read_csv requires a type specification as string");
 				}
-				return_types.push_back(TransformStringToLogicalType(val.second.str_value.c_str()));
+				return_types.emplace_back(TransformStringToLogicalType(val.str_value.c_str()));
 			}
 			if (names.empty()) {
 				throw BinderException("read_csv requires at least a single column as input!");
@@ -149,7 +152,8 @@ struct ReadCSVOperatorData : public FunctionOperatorData {
 };
 
 static unique_ptr<FunctionOperatorData> ReadCSVInit(ClientContext &context, const FunctionData *bind_data_p,
-                                                    vector<column_t> &column_ids, TableFilterCollection *filters) {
+                                                    const vector<column_t> &column_ids,
+                                                    TableFilterCollection *filters) {
 	auto &bind_data = (ReadCSVData &)*bind_data_p;
 	auto result = make_unique<ReadCSVOperatorData>();
 	if (bind_data.initial_reader) {
