@@ -13,6 +13,7 @@
 #include "duckdb/common/vector.hpp"
 #include "duckdb/common/unordered_map.hpp"
 #include "duckdb/common/exception.hpp"
+#include "duckdb/common/enums/file_compression_type.hpp"
 
 #include <functional>
 
@@ -33,10 +34,20 @@ public:
 	virtual ~FileHandle() {
 	}
 
+	int64_t Read(void *buffer, idx_t nr_bytes);
+	int64_t Write(void *buffer, idx_t nr_bytes);
 	void Read(void *buffer, idx_t nr_bytes, idx_t location);
 	void Write(void *buffer, idx_t nr_bytes, idx_t location);
+	void Seek(idx_t location);
+	void Reset();
+	idx_t SeekPosition();
 	void Sync();
 	void Truncate(int64_t new_size);
+	string ReadLine();
+
+	bool CanSeek();
+	bool OnDiskFile();
+	idx_t GetFileSize();
 
 protected:
 	virtual void Close() = 0;
@@ -73,10 +84,10 @@ public:
 	static FileSystem &GetFileSystem(ClientContext &context);
 	static FileSystem &GetFileSystem(DatabaseInstance &db);
 
-	virtual unique_ptr<FileHandle> OpenFile(const char *path, uint8_t flags, FileLockType lock = FileLockType::NO_LOCK);
-	unique_ptr<FileHandle> OpenFile(string &path, uint8_t flags, FileLockType lock = FileLockType::NO_LOCK) {
-		return OpenFile(path.c_str(), flags, lock);
-	}
+	virtual unique_ptr<FileHandle> OpenFile(const string &path, uint8_t flags,
+	                                        FileLockType lock = FileLockType::NO_LOCK,
+	                                        FileCompressionType compression = FileCompressionType::UNCOMPRESSED);
+
 	//! Read exactly nr_bytes from the specified location in the file. Fails if nr_bytes could not be read. This is
 	//! equivalent to calling SetFilePointer(location) followed by calling Read().
 	virtual void Read(FileHandle &handle, void *buffer, int64_t nr_bytes, idx_t location);
@@ -137,22 +148,38 @@ public:
 	virtual idx_t GetAvailableMemory();
 
 	//! registers a sub-file system to handle certain file name prefixes, e.g. http:// etc.
-	virtual void RegisterProtocolHandler(string protocol, unique_ptr<FileSystem> protocol_fs) {
-		throw NotImplementedException("Can't register a protocol handler on a non-virtual file system");
+	virtual void RegisterSubSystem(unique_ptr<FileSystem> sub_fs) {
+		throw NotImplementedException("Can't register a sub system on a non-virtual file system");
 	}
 
+	virtual bool CanHandleFile(const string &fpath) {
+		//! Whether or not a sub-system can handle a specific file path
+		return false;
+	}
+
+	//! Set the file pointer of a file handle to a specified location. Reads and writes will happen from this location
+	virtual void Seek(FileHandle &handle, idx_t location);
+	//! Reset a file to the beginning (equivalent to Seek(handle, 0) for simple files)
+	virtual void Reset(FileHandle &handle);
+	virtual idx_t SeekPosition(FileHandle &handle);
+
+	//! Whether or not we can seek into the file
+	virtual bool CanSeek();
+	//! Whether or not the FS handles plain files on disk. This is relevant for certain optimizations, as random reads
+	//! in a file on-disk are much cheaper than e.g. random reads in a file over the network
+	virtual bool OnDiskFile(FileHandle &handle);
+
 private:
 	//! Set the file pointer of a file handle to a specified location. Reads and writes will happen from this location
 	void SetFilePointer(FileHandle &handle, idx_t location);
+	virtual idx_t GetFilePointer(FileHandle &handle);
 };
 
 // bunch of wrappers to allow registering protocol handlers
 class VirtualFileSystem : public FileSystem {
 public:
-	unique_ptr<FileHandle> OpenFile(const char *path, uint8_t flags,
-	                                FileLockType lock = FileLockType::NO_LOCK) override {
-		return FindFileSystem(path)->OpenFile(path, flags, lock);
-	}
+	unique_ptr<FileHandle> OpenFile(const string &path, uint8_t flags, FileLockType lock = FileLockType::NO_LOCK,
+	                                FileCompressionType compression = FileCompressionType::UNCOMPRESSED) override;
 
 	virtual void Read(FileHandle &handle, void *buffer, int64_t nr_bytes, idx_t location) override {
 		handle.file_system.Read(handle, buffer, nr_bytes, location);
@@ -234,22 +261,22 @@ public:
 		return default_fs.GetAvailableMemory();
 	}
 
-	void RegisterProtocolHandler(string protocol, unique_ptr<FileSystem> protocol_fs) override {
-		protocol_handler_fss[protocol] = move(protocol_fs);
+	void RegisterSubSystem(unique_ptr<FileSystem> fs) override {
+		sub_systems.push_back(move(fs));
 	}
 
 private:
 	FileSystem *FindFileSystem(const string &path) {
-		for (auto &handler : protocol_handler_fss) {
-			if (path.rfind(handler.first, 0) == 0) {
-				return handler.second.get();
+		for (auto &sub_system : sub_systems) {
+			if (sub_system->CanHandleFile(path)) {
+				return sub_system.get();
 			}
 		}
 		return &default_fs;
 	}
 
 private:
-	unordered_map<string, unique_ptr<FileSystem>> protocol_handler_fss;
+	vector<unique_ptr<FileSystem>> sub_systems;
 	FileSystem default_fs;
 };
 
