@@ -27,6 +27,9 @@ public:
 	static bool CharacterIsNewline(char c) {
 		return c == '\n' || c == '\r';
 	}
+	static bool CharacterIsDigit(char c) {
+		return c >= '0' && c <= '9';
+	}
 
 	//! Returns true if the needle string exists in the haystack
 	static bool Contains(const string &haystack, const string &needle);
@@ -80,7 +83,8 @@ public:
 	static string Lower(const string &str);
 
 	//! Format a string using printf semantics
-	template <typename... Args> static string Format(const string fmt_str, Args... params) {
+	template <typename... Args>
+	static string Format(const string fmt_str, Args... params) {
 		return Exception::ConstructMessage(fmt_str, params...);
 	}
 
@@ -105,8 +109,8 @@ public:
 	static vector<string> TopNStrings(vector<std::pair<string, idx_t>> scores, idx_t n = 5, idx_t threshold = 5);
 	//! Computes the levenshtein distance of each string in strings, and compares it to target, then returns TopNStrings
 	//! with the given params.
-	static vector<string> TopNLevenshtein(vector<string> strings, const string &target, idx_t n = 5,
+	static vector<string> TopNLevenshtein(const vector<string> &strings, const string &target, idx_t n = 5,
 	                                      idx_t threshold = 5);
-	static string CandidatesMessage(const vector<string> &candidates, string candidate = "Candidate bindings");
+	static string CandidatesMessage(const vector<string> &candidates, const string &candidate = "Candidate bindings");
 };
 } // namespace duckdb
