@@ -27,7 +27,8 @@ inline hash_t murmurhash32(uint32_t x) {
 	return murmurhash64(x);
 }
 
-template <class T> hash_t Hash(T value) {
+template <class T>
+hash_t Hash(T value) {
 	return murmurhash32(value);
 }
 
@@ -36,15 +37,24 @@ inline hash_t CombineHash(hash_t left, hash_t right) {
 	return left ^ right;
 }
 
-template <> hash_t Hash(uint64_t val);
-template <> hash_t Hash(int64_t val);
-template <> hash_t Hash(hugeint_t val);
-template <> hash_t Hash(float val);
-template <> hash_t Hash(double val);
-template <> hash_t Hash(const char *val);
-template <> hash_t Hash(char *val);
-template <> hash_t Hash(string_t val);
-template <> hash_t Hash(interval_t val);
+template <>
+hash_t Hash(uint64_t val);
+template <>
+hash_t Hash(int64_t val);
+template <>
+hash_t Hash(hugeint_t val);
+template <>
+hash_t Hash(float val);
+template <>
+hash_t Hash(double val);
+template <>
+hash_t Hash(const char *val);
+template <>
+hash_t Hash(char *val);
+template <>
+hash_t Hash(string_t val);
+template <>
+hash_t Hash(interval_t val);
 hash_t Hash(const char *val, size_t size);
 hash_t Hash(char *val, size_t size);
 hash_t Hash(uint8_t *val, size_t size);
