@@ -11,9 +11,11 @@
 #include "duckdb/common/exception.hpp"
 #include "duckdb/common/types/vector.hpp"
 #include "duckdb/common/vector_operations/vector_operations.hpp"
+#include "duckdb/function/aggregate_function.hpp"
 
 namespace duckdb {
 struct FunctionData;
+typedef std::pair<idx_t, idx_t> FrameBounds;
 
 class AggregateExecutor {
 private:
@@ -301,7 +303,7 @@ public:
 	template <class STATE_TYPE, class OP>
 	static void Combine(Vector &source, Vector &target, idx_t count) {
 		D_ASSERT(source.GetType().id() == LogicalTypeId::POINTER && target.GetType().id() == LogicalTypeId::POINTER);
-		auto sdata = FlatVector::GetData<STATE_TYPE *>(source);
+		auto sdata = FlatVector::GetData<const STATE_TYPE *>(source);
 		auto tdata = FlatVector::GetData<STATE_TYPE *>(target);
 
 		for (idx_t i = 0; i < count; i++) {
@@ -331,6 +333,18 @@ public:
 		}
 	}
 
+	template <class STATE, class INPUT_TYPE, class RESULT_TYPE, class OP>
+	static void UnaryWindow(Vector &input, FunctionData *bind_data, data_ptr_t state, const FrameBounds &frame,
+	                        const FrameBounds &prev, Vector &result) {
+
+		auto idata = FlatVector::GetData<const INPUT_TYPE>(input) - MinValue(frame.first, prev.first);
+		const auto &ivalid = FlatVector::Validity(input);
+		auto rdata = ConstantVector::GetData<RESULT_TYPE>(result);
+		auto &rvalid = ConstantVector::Validity(result);
+		OP::template Window<STATE, INPUT_TYPE, RESULT_TYPE>(idata, ivalid, bind_data, (STATE *)state, frame, prev,
+		                                                    rdata, rvalid);
+	}
+
 	template <class STATE_TYPE, class OP>
 	static void Destroy(Vector &states, idx_t count) {
 		auto sdata = FlatVector::GetData<STATE_TYPE *>(states);
