@@ -13,6 +13,7 @@
 namespace duckdb {
 class BlockHandle;
 class BufferHandle;
+class RowDataCollection;
 
 //! GroupedAggregateHashTable is a linear probing HT that is used for computing
 //! aggregates
@@ -33,9 +34,10 @@ class BufferHandle;
 // NOTE: PAGE_NR and PAGE_OFFSET are reversed for 64 bit HTs because struct packing
 
 // payload layout
-// [HASH][GROUPS][PADDING][PAYLOAD]
-// [HASH] is the hash of the groups
+// [VALIDITY][GROUPS][HASH][PADDING][PAYLOAD]
+// [VALIDITY] is the validity bits of the data columns (including the HASH)
 // [GROUPS] is the group data, could be multiple values, fixed size, strings are elsewhere
+// [HASH] is the hash data of the groups
 // [PADDING] is gunk data to align payload properly
 // [PAYLOAD] is the payload (i.e. the aggregate states)
 struct aggr_ht_entry_64 {
@@ -86,8 +88,6 @@ public:
 	void FindOrCreateGroups(DataChunk &groups, Vector &addresses_out);
 
 	//! Executes the filter(if any) and update the aggregates
-	static void UpdateAggregate(AggregateObject &aggr, DataChunk &payload, Vector &distinct_addresses,
-	                            idx_t input_count, idx_t payload_idx);
 	void Combine(GroupedAggregateHashTable &other);
 
 	idx_t Size() {
@@ -101,7 +101,7 @@ public:
 	void Finalize();
 
 	//! The stringheap of the AggregateHashTable
-	StringHeap string_heap;
+	unique_ptr<RowDataCollection> string_heap;
 
 	//! The hash table load factor, when a resize is triggered
 	constexpr static float LOAD_FACTOR = 1.5;
@@ -127,6 +127,7 @@ private:
 	unique_ptr<BufferHandle> hashes_hdl;
 	data_ptr_t hashes_hdl_ptr;
 	data_ptr_t hashes_end_ptr; // of hashes
+	idx_t hash_offset;         // Offset into the layout of the hash column
 
 	idx_t hash_prefix_shift;
 	idx_t payload_page_offset;
@@ -147,6 +148,7 @@ private:
 	SelectionVector group_compare_vector;
 	SelectionVector no_match_vector;
 	SelectionVector empty_vector;
+	vector<ExpressionType> predicates;
 
 private:
 	GroupedAggregateHashTable(const GroupedAggregateHashTable &) = delete;
@@ -154,19 +156,17 @@ private:
 	//! Resize the HT to the specified size. Must be larger than the current
 	//! size.
 	void Destroy();
-	void ScatterGroups(DataChunk &groups, unique_ptr<VectorData[]> &group_data, Vector &addresses,
-	                   const SelectionVector &sel, idx_t count);
 
 	void Verify();
 
 	void FlushMove(Vector &source_addresses, Vector &source_hashes, idx_t count);
 	void NewBlock();
 
-	template <class T>
+	template <class ENTRY>
 	void VerifyInternal();
-	template <class T>
+	template <class ENTRY>
 	void Resize(idx_t size);
-	template <class T>
+	template <class ENTRY>
 	idx_t FindOrCreateGroupsInternal(DataChunk &groups, Vector &group_hashes, Vector &addresses,
 	                                 SelectionVector &new_groups);
 
