@@ -10,6 +10,7 @@
 
 #include "duckdb/common/common.hpp"
 #include "duckdb/common/types/data_chunk.hpp"
+#include "duckdb/common/types/row_layout.hpp"
 #include "duckdb/common/types/vector.hpp"
 #include "duckdb/execution/aggregate_hashtable.hpp"
 #include "duckdb/planner/operator/logical_comparison_join.hpp"
@@ -18,6 +19,7 @@
 namespace duckdb {
 class BufferManager;
 class BufferHandle;
+class RowDataCollection;
 
 struct JoinHTScanState {
 	JoinHTScanState() : position(0), block_position(0) {
@@ -43,6 +45,8 @@ struct JoinHTScanState {
 */
 class JoinHashTable {
 public:
+	using ValidityBytes = TemplatedValidityMask<uint8_t>;
+
 	//! Scan structure that can be used to resume scans, as a single probe can
 	//! return 1024*N values (where N is the size of the HT). This is
 	//! returned by the JoinHashTable::Scan function and can be used to resume a
@@ -88,14 +92,11 @@ public:
 
 		idx_t ScanInnerJoin(DataChunk &keys, SelectionVector &result_vector);
 
-		idx_t ResolvePredicates(DataChunk &keys, SelectionVector &match_sel);
-		idx_t ResolvePredicates(DataChunk &keys, SelectionVector &match_sel, SelectionVector &no_match_sel);
 		void GatherResult(Vector &result, const SelectionVector &result_vector, const SelectionVector &sel_vector,
-		                  idx_t count, idx_t &offset);
-		void GatherResult(Vector &result, const SelectionVector &sel_vector, idx_t count, idx_t &offset);
+		                  const idx_t count, const idx_t col_idx);
+		void GatherResult(Vector &result, const SelectionVector &sel_vector, const idx_t count, const idx_t col_idx);
 
-		template <bool NO_MATCH_SEL>
-		idx_t ResolvePredicates(DataChunk &keys, SelectionVector *match_sel, SelectionVector *no_match_sel);
+		idx_t ResolvePredicates(DataChunk &keys, SelectionVector &match_sel, SelectionVector *no_match_sel);
 	};
 
 private:
@@ -141,7 +142,7 @@ public:
 	}
 
 	//! The stringheap of the JoinHashTable
-	StringHeap string_heap;
+	unique_ptr<RowDataCollection> string_heap;
 	//! BufferManager
 	BufferManager &buffer_manager;
 	//! The types of the keys used in equality comparison
@@ -152,18 +153,18 @@ public:
 	vector<LogicalType> build_types;
 	//! The comparison predicates
 	vector<ExpressionType> predicates;
-	//! Size of condition keys
-	idx_t equality_size;
-	//! Size of condition keys
-	idx_t condition_size;
-	//! Size of build tuple
-	idx_t build_size;
+	//! Data column layout
+	RowLayout layout;
+	//! Size of the validity vector for each tuple.
+	idx_t validity_size;
 	//! The size of an entry as stored in the HashTable
 	idx_t entry_size;
 	//! The total tuple size
 	idx_t tuple_size;
 	//! Next pointer offset in tuple
 	idx_t pointer_offset;
+	//! A constant false column for initialising right outer joins
+	Vector vfound;
 	//! The join type of the HT
 	JoinType join_type;
 	//! Whether or not the HT has been finalized
@@ -202,9 +203,6 @@ private:
 
 	idx_t PrepareKeys(DataChunk &keys, unique_ptr<VectorData[]> &key_data, const SelectionVector *&current_sel,
 	                  SelectionVector &sel, bool build_side);
-	void SerializeVectorData(VectorData &vdata, PhysicalType type, const SelectionVector &sel, idx_t count,
-	                         data_ptr_t key_locations[]);
-	void SerializeVector(Vector &v, idx_t vcount, const SelectionVector &sel, idx_t count, data_ptr_t key_locations[]);
 
 	//! The amount of entries stored in the HT currently
 	idx_t count;
