@@ -20,13 +20,14 @@ public:
 	      offset(offset), limit_expression(move(limit_expression)), offset_expression(move(offset_expression)) {
 	}
 
-	idx_t limit;
-	idx_t offset;
-	unique_ptr<Expression> limit_expression;
-	unique_ptr<Expression> offset_expression;
+	// Thread safety
+	mutable idx_t limit;
+	mutable idx_t offset;
+	mutable unique_ptr<Expression> limit_expression;
+	mutable unique_ptr<Expression> offset_expression;
 
 public:
-	void GetChunkInternal(ExecutionContext &context, DataChunk &chunk, PhysicalOperatorState *state) override;
+	void GetChunkInternal(ExecutionContext &context, DataChunk &chunk, PhysicalOperatorState *state) const override;
 
 	unique_ptr<PhysicalOperatorState> GetOperatorState() override;
 };
