@@ -9,7 +9,6 @@
 #pragma once
 
 #include "duckdb/execution/physical_operator.hpp"
-#include "duckdb/common/types/chunk_collection.hpp"
 
 namespace duckdb {
 class Pipeline;
@@ -22,19 +21,18 @@ public:
 
 	bool union_all;
 	std::shared_ptr<ChunkCollection> working_table;
-	ChunkCollection intermediate_table;
-	vector<unique_ptr<Pipeline>> pipelines;
+	vector<shared_ptr<Pipeline>> pipelines;
 
 public:
-	void GetChunkInternal(ExecutionContext &context, DataChunk &chunk, PhysicalOperatorState *state) override;
+	void GetChunkInternal(ExecutionContext &context, DataChunk &chunk, PhysicalOperatorState *state) const override;
 	unique_ptr<PhysicalOperatorState> GetOperatorState() override;
 	void FinalizeOperatorState(PhysicalOperatorState &state_p, ExecutionContext &context) override;
 
 private:
 	//! Probe Hash Table and eliminate duplicate rows
-	idx_t ProbeHT(DataChunk &chunk, PhysicalOperatorState *state);
+	idx_t ProbeHT(DataChunk &chunk, PhysicalOperatorState *state) const;
 
-	void ExecuteRecursivePipelines(ExecutionContext &context);
+	void ExecuteRecursivePipelines(ExecutionContext &context) const;
 };
 
 } // namespace duckdb
