@@ -15,7 +15,7 @@ namespace duckdb {
 // The Conjunction Simplification rule rewrites conjunctions with a constant
 class ConjunctionSimplificationRule : public Rule {
 public:
-	ConjunctionSimplificationRule(ExpressionRewriter &rewriter);
+	explicit ConjunctionSimplificationRule(ExpressionRewriter &rewriter);
 
 	unique_ptr<Expression> Apply(LogicalOperator &op, vector<Expression *> &bindings, bool &changes_made) override;
 
