@@ -16,7 +16,7 @@ namespace duckdb {
 // (X AND B) OR (X AND C) OR (X AND D) = X AND (B OR C OR D)
 class DistributivityRule : public Rule {
 public:
-	DistributivityRule(ExpressionRewriter &rewriter);
+	explicit DistributivityRule(ExpressionRewriter &rewriter);
 
 	unique_ptr<Expression> Apply(LogicalOperator &op, vector<Expression *> &bindings, bool &changes_made) override;
 
