@@ -16,8 +16,8 @@ namespace duckdb {
 //! LogicalOrder represents an ORDER BY clause, sorting the data
 class LogicalOrder : public LogicalOperator {
 public:
-	LogicalOrder(vector<BoundOrderByNode> orders)
-	    : LogicalOperator(LogicalOperatorType::ORDER_BY), orders(move(orders)) {
+	explicit LogicalOrder(vector<BoundOrderByNode> orders)
+	    : LogicalOperator(LogicalOperatorType::LOGICAL_ORDER_BY), orders(move(orders)) {
 	}
 
 	vector<BoundOrderByNode> orders;
