@@ -13,7 +13,7 @@
 namespace duckdb {
 class ColumnData;
 class UncompressedSegment;
-class MorselInfo;
+class RowGroup;
 class BaseStatistics;
 class SegmentStatistics;
 
@@ -25,10 +25,7 @@ public:
 	TableDataWriter(DatabaseInstance &db, TableCatalogEntry &table, MetaBlockWriter &meta_writer);
 	~TableDataWriter();
 
-	void WriteTableData();
-
-	void CheckpointColumn(ColumnData &col_data, idx_t col_idx);
-	void CheckpointDeletes(MorselInfo *info);
+	BlockPointer WriteTableData();
 
 	MetaBlockWriter &GetMetaWriter() {
 		return meta_writer;
