@@ -19,10 +19,16 @@ public:
 
 	//! The size of this type
 	idx_t type_size;
+	//! The maximum amount of tuples that will fit in this segment
+	idx_t max_tuple_count;
 
 public:
 	void InitializeScan(ColumnScanState &state) override;
 
+	//! Scans a vector of "scan_count" entries starting at position "start"
+	//! Store it in result with offset "result_offset"
+	void Scan(ColumnScanState &state, idx_t start, idx_t scan_count, Vector &result, idx_t result_offset) override;
+
 	//! Fetch a single value and append it to the vector
 	void FetchRow(ColumnFetchState &state, row_t row_id, Vector &result, idx_t result_idx) override;
 
@@ -31,9 +37,6 @@ public:
 	//! full.
 	idx_t Append(SegmentStatistics &stats, VectorData &data, idx_t offset, idx_t count) override;
 
-protected:
-	void FetchBaseData(ColumnScanState &state, idx_t vector_index, Vector &result) override;
-
 public:
 	typedef void (*append_function_t)(SegmentStatistics &stats, data_ptr_t target, idx_t target_offset,
 	                                  VectorData &source, idx_t offset, idx_t count);
