@@ -8,24 +8,19 @@
 
 #pragma once
 
-#include "duckdb/storage/column_data.hpp"
+#include "duckdb/storage/table/column_data.hpp"
 
 namespace duckdb {
 
 //! Validity column data represents the validity data (i.e. which values are null)
 class ValidityColumnData : public ColumnData {
 public:
-	ValidityColumnData(DatabaseInstance &db, DataTableInfo &table_info, idx_t column_idx);
+	ValidityColumnData(DataTableInfo &info, idx_t column_index, idx_t start_row, ColumnData *parent);
 
 public:
 	bool CheckZonemap(ColumnScanState &state, TableFilter &filter) override;
 	void InitializeScan(ColumnScanState &state) override;
-	void InitializeScanWithOffset(ColumnScanState &state, idx_t vector_idx) override;
-	void Scan(Transaction &transaction, ColumnScanState &state, Vector &result) override;
-	void IndexScan(ColumnScanState &state, Vector &result, bool allow_pending_updates) override;
-	void Update(Transaction &transaction, Vector &updates, Vector &row_ids, idx_t count) override;
-
-	static unique_ptr<PersistentColumnData> Deserialize(DatabaseInstance &db, Deserializer &source);
+	void InitializeScanWithOffset(ColumnScanState &state, idx_t row_idx) override;
 };
 
 } // namespace duckdb
