@@ -16,8 +16,10 @@
 namespace duckdb {
 class BlockHandle;
 class ColumnData;
+class ConstantFilter;
 class Transaction;
 class StorageManager;
+class TableFilter;
 
 struct ColumnAppendState;
 struct UpdateInfo;
@@ -34,10 +36,6 @@ public:
 	PhysicalType type;
 	//! The block that this segment relates to
 	shared_ptr<BlockHandle> block;
-	//! The size of a vector of this type
-	idx_t vector_size;
-	//! The maximum amount of vectors that can be stored in this segment
-	idx_t max_vector_count;
 	//! The current amount of tuples that are stored in this segment
 	atomic<idx_t> tuple_count;
 	//! The starting row of this segment
@@ -46,14 +44,13 @@ public:
 public:
 	virtual void InitializeScan(ColumnScanState &state) {
 	}
-	//! Fetch the vector at index "vector_index" from the uncompressed segment, storing it in the result vector
-	void Scan(ColumnScanState &state, idx_t vector_index, Vector &result);
+	//! Scans a vector of "scan_count" entries starting at position "start"
+	//! Store it in result with offset "result_offset"
+	virtual void Scan(ColumnScanState &state, idx_t start, idx_t scan_count, Vector &result, idx_t result_offset) = 0;
 
 	static void FilterSelection(SelectionVector &sel, Vector &result, const TableFilter &filter,
 	                            idx_t &approved_tuple_count, ValidityMask &mask);
 
-	//! Fetch a single vector from the base table
-	void Fetch(ColumnScanState &state, idx_t vector_index, Vector &result);
 	//! Fetch a single value and append it to the vector
 	virtual void FetchRow(ColumnFetchState &state, row_t row_id, Vector &result, idx_t result_idx) = 0;
 
@@ -64,23 +61,10 @@ public:
 	//! Truncate a previous append
 	virtual void RevertAppend(idx_t start_row);
 
-	//! Convert a persistently backed uncompressed segment (i.e. one where block_id refers to an on-disk block) to a
-	//! temporary in-memory one
-	virtual void ToTemporary();
-	void ToTemporaryInternal();
-
-	//! Get the amount of tuples in a vector
-	idx_t GetVectorCount(idx_t vector_index) {
-		D_ASSERT(vector_index < max_vector_count);
-		D_ASSERT(vector_index * STANDARD_VECTOR_SIZE <= tuple_count);
-		return MinValue<idx_t>(STANDARD_VECTOR_SIZE, tuple_count - vector_index * STANDARD_VECTOR_SIZE);
-	}
-
 	virtual void Verify();
 
-protected:
-	//! Fetch base table data
-	virtual void FetchBaseData(ColumnScanState &state, idx_t vector_index, Vector &result) = 0;
+	bool RowIdIsValid(idx_t row_id) const;
+	bool RowRangeIsValid(idx_t row_id, idx_t count) const;
 };
 
 } // namespace duckdb
