@@ -1,5 +1,5 @@
 #include "duckdb/main/client_context.hpp"
-
+#include "duckdb/main/query_profiler.hpp"
 #include "duckdb/catalog/catalog_entry/table_catalog_entry.hpp"
 #include "duckdb/common/serializer/buffered_deserializer.hpp"
 #include "duckdb/common/serializer/buffered_serializer.hpp"
@@ -44,10 +44,13 @@ private:
 };
 
 ClientContext::ClientContext(shared_ptr<DatabaseInstance> database)
-    : db(move(database)), transaction(db->GetTransactionManager(), *this), interrupted(false), executor(*this),
+    : profiler(make_unique<QueryProfiler>()), query_profiler_history(make_unique<QueryProfilerHistory>()),
+      db(move(database)), transaction(db->GetTransactionManager(), *this), interrupted(false), executor(*this),
       temporary_objects(make_unique<SchemaCatalogEntry>(&db->GetCatalog(), TEMP_SCHEMA, true)), open_result(nullptr) {
 	std::random_device rd;
 	random_engine.seed(rd());
+
+	progress_bar = make_unique<ProgressBar>(&executor, wait_time);
 }
 
 ClientContext::~ClientContext() {
@@ -95,17 +98,21 @@ unique_ptr<DataChunk> ClientContext::Fetch() {
 }
 
 string ClientContext::FinalizeQuery(ClientContextLock &lock, bool success) {
-	profiler.EndQuery();
+	profiler->EndQuery();
 	executor.Reset();
 
 	string error;
 	if (transaction.HasActiveTransaction()) {
 		ActiveTransaction().active_query = MAXIMUM_QUERY_ID;
-		query_profiler_history.GetPrevProfilers().emplace_back(transaction.ActiveTransaction().active_query,
-		                                                       move(profiler));
-		profiler.save_location = query_profiler_history.GetPrevProfilers().back().second.save_location;
-		if (query_profiler_history.GetPrevProfilers().size() >= query_profiler_history.GetPrevProfilersSize()) {
-			query_profiler_history.GetPrevProfilers().pop_front();
+		// Move the query profiler into the history
+		auto &prev_profilers = query_profiler_history->GetPrevProfilers();
+		prev_profilers.emplace_back(transaction.ActiveTransaction().active_query, move(profiler));
+		// Reinitialize the query profiler
+		profiler = make_unique<QueryProfiler>();
+		// Propagate settings of the saved query into the new profiler.
+		profiler->Propagate(*prev_profilers.back().second);
+		if (prev_profilers.size() >= query_profiler_history->GetPrevProfilersSize()) {
+			prev_profilers.pop_front();
 		}
 		try {
 			if (transaction.IsAutoCommit()) {
@@ -154,11 +161,11 @@ shared_ptr<PreparedStatementData> ClientContext::CreatePreparedStatement(ClientC
 	StatementType statement_type = statement->type;
 	auto result = make_shared<PreparedStatementData>(statement_type);
 
-	profiler.StartPhase("planner");
+	profiler->StartPhase("planner");
 	Planner planner(*this);
 	planner.CreatePlan(move(statement));
 	D_ASSERT(planner.plan);
-	profiler.EndPhase();
+	profiler->EndPhase();
 
 	auto plan = move(planner.plan);
 	// extract the result column names from the plan
@@ -171,29 +178,29 @@ shared_ptr<PreparedStatementData> ClientContext::CreatePreparedStatement(ClientC
 	result->catalog_version = Transaction::GetTransaction(*this).catalog_version;
 
 	if (enable_optimizer) {
-		profiler.StartPhase("optimizer");
+		profiler->StartPhase("optimizer");
 		Optimizer optimizer(*planner.binder, *this);
 		plan = optimizer.Optimize(move(plan));
 		D_ASSERT(plan);
-		profiler.EndPhase();
+		profiler->EndPhase();
 	}
 
-	profiler.StartPhase("physical_planner");
+	profiler->StartPhase("physical_planner");
 	// now convert logical query plan into a physical query plan
 	PhysicalPlanGenerator physical_planner(*this);
 	auto physical_plan = physical_planner.CreatePlan(move(plan));
-	profiler.EndPhase();
+	profiler->EndPhase();
 
 	result->plan = move(physical_plan);
 	return result;
 }
 
 int ClientContext::GetProgress() {
-	auto my_progress_bar = progress_bar;
-	if (!my_progress_bar) {
+	//	auto my_progress_bar = progress_bar;
+	if (!progress_bar) {
 		return -1;
 	}
-	return my_progress_bar->GetCurrentPercentage();
+	return progress_bar->GetCurrentPercentage();
 }
 
 unique_ptr<QueryResult> ClientContext::ExecutePreparedStatement(ClientContextLock &lock, const string &query,
@@ -214,9 +221,7 @@ unique_ptr<QueryResult> ClientContext::ExecutePreparedStatement(ClientContextLoc
 
 	bool create_stream_result = statement.allow_stream_result && allow_stream_result;
 	if (enable_progress_bar) {
-		if (!progress_bar) {
-			progress_bar = make_shared<ProgressBar>(&executor, wait_time);
-		}
+		progress_bar->Initialize(wait_time);
 		progress_bar->Start();
 	}
 	// store the physical plan in the context for calls to Fetch()
@@ -227,7 +232,7 @@ unique_ptr<QueryResult> ClientContext::ExecutePreparedStatement(ClientContextLoc
 	D_ASSERT(types == statement.types);
 
 	if (create_stream_result) {
-		if (progress_bar) {
+		if (enable_progress_bar) {
 			progress_bar->Stop();
 		}
 		// successfully compiled SELECT clause and it is the last statement
@@ -251,7 +256,7 @@ unique_ptr<QueryResult> ClientContext::ExecutePreparedStatement(ClientContextLoc
 #endif
 		result->collection.Append(*chunk);
 	}
-	if (progress_bar) {
+	if (enable_progress_bar) {
 		progress_bar->Stop();
 	}
 	return move(result);
@@ -382,7 +387,7 @@ unique_ptr<QueryResult> ClientContext::RunStatementOrPreparedStatement(ClientCon
 		statement = move(copied_statement);
 	}
 	// start the profiler
-	profiler.StartQuery(query);
+	profiler->StartQuery(query);
 	try {
 		if (statement) {
 			result = RunStatementInternal(lock, query, move(statement), allow_stream_result);
@@ -510,12 +515,12 @@ void ClientContext::Interrupt() {
 
 void ClientContext::EnableProfiling() {
 	auto lock = LockContext();
-	profiler.Enable();
+	profiler->Enable();
 }
 
 void ClientContext::DisableProfiling() {
 	auto lock = LockContext();
-	profiler.Disable();
+	profiler->Disable();
 }
 
 string ClientContext::VerifyQuery(ClientContextLock &lock, const string &query, unique_ptr<SQLStatement> statement) {
@@ -575,9 +580,9 @@ string ClientContext::VerifyQuery(ClientContextLock &lock, const string &query,
 #endif
 
 	// disable profiling if it is enabled
-	bool profiling_is_enabled = profiler.IsEnabled();
+	bool profiling_is_enabled = profiler->IsEnabled();
 	if (profiling_is_enabled) {
-		profiler.Disable();
+		profiler->Disable();
 	}
 
 	// see below
@@ -644,7 +649,7 @@ string ClientContext::VerifyQuery(ClientContextLock &lock, const string &query,
 	enable_optimizer = true;
 
 	if (profiling_is_enabled) {
-		profiler.Enable();
+		profiler->Enable();
 	}
 
 	// now compare the results
@@ -686,25 +691,26 @@ void ClientContext::RunFunctionInTransactionInternal(ClientContextLock &lock, co
 		throw Exception("Failed: transaction has been invalidated!");
 	}
 	// check if we are on AutoCommit. In this case we should start a transaction
-	if (transaction.IsAutoCommit()) {
+	bool require_new_transaction = transaction.IsAutoCommit() && !transaction.HasActiveTransaction();
+	if (require_new_transaction) {
 		transaction.BeginTransaction();
 	}
 	try {
 		fun();
 	} catch (StandardException &ex) {
-		if (transaction.IsAutoCommit()) {
+		if (require_new_transaction) {
 			transaction.Rollback();
 		}
 		throw;
 	} catch (std::exception &ex) {
-		if (transaction.IsAutoCommit()) {
+		if (require_new_transaction) {
 			transaction.Rollback();
 		} else {
 			ActiveTransaction().Invalidate();
 		}
 		throw;
 	}
-	if (transaction.IsAutoCommit()) {
+	if (require_new_transaction) {
 		transaction.Commit();
 	}
 }
