@@ -1,5 +1,5 @@
 #include "duckdb/optimizer/optimizer.hpp"
-
+#include "duckdb/main/query_profiler.hpp"
 #include "duckdb/execution/expression_executor.hpp"
 #include "duckdb/main/client_context.hpp"
 #include "duckdb/optimizer/column_lifetime_optimizer.hpp"
@@ -46,83 +46,83 @@ Optimizer::Optimizer(Binder &binder, ClientContext &context) : context(context),
 unique_ptr<LogicalOperator> Optimizer::Optimize(unique_ptr<LogicalOperator> plan) {
 	// first we perform expression rewrites using the ExpressionRewriter
 	// this does not change the logical plan structure, but only simplifies the expression trees
-	context.profiler.StartPhase("expression_rewriter");
+	context.profiler->StartPhase("expression_rewriter");
 	rewriter.VisitOperator(*plan);
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
 	// perform filter pullup
-	context.profiler.StartPhase("filter_pullup");
+	context.profiler->StartPhase("filter_pullup");
 	FilterPullup filter_pullup;
 	plan = filter_pullup.Rewrite(move(plan));
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
 	// perform filter pushdown
-	context.profiler.StartPhase("filter_pushdown");
+	context.profiler->StartPhase("filter_pushdown");
 	FilterPushdown filter_pushdown(*this);
 	plan = filter_pushdown.Rewrite(move(plan));
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
-	context.profiler.StartPhase("regex_range");
+	context.profiler->StartPhase("regex_range");
 	RegexRangeFilter regex_opt;
 	plan = regex_opt.Rewrite(move(plan));
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
-	context.profiler.StartPhase("in_clause");
+	context.profiler->StartPhase("in_clause");
 	InClauseRewriter rewriter(*this);
 	plan = rewriter.Rewrite(move(plan));
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
 	// then we perform the join ordering optimization
 	// this also rewrites cross products + filters into joins and performs filter pushdowns
-	context.profiler.StartPhase("join_order");
+	context.profiler->StartPhase("join_order");
 	JoinOrderOptimizer optimizer(context);
 	plan = optimizer.Optimize(move(plan));
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
 	// removes any redundant DelimGets/DelimJoins
-	context.profiler.StartPhase("deliminator");
+	context.profiler->StartPhase("deliminator");
 	Deliminator deliminator;
 	plan = deliminator.Optimize(move(plan));
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
-	context.profiler.StartPhase("unused_columns");
+	context.profiler->StartPhase("unused_columns");
 	RemoveUnusedColumns unused(binder, context, true);
 	unused.VisitOperator(*plan);
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
 	// perform statistics propagation
-	context.profiler.StartPhase("statistics_propagation");
+	context.profiler->StartPhase("statistics_propagation");
 	StatisticsPropagator propagator(context);
 	propagator.PropagateStatistics(plan);
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
 	// then we extract common subexpressions inside the different operators
-	context.profiler.StartPhase("common_subexpressions");
+	context.profiler->StartPhase("common_subexpressions");
 	CommonSubExpressionOptimizer cse_optimizer(binder);
 	cse_optimizer.VisitOperator(*plan);
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
-	context.profiler.StartPhase("common_aggregate");
+	context.profiler->StartPhase("common_aggregate");
 	CommonAggregateOptimizer common_aggregate;
 	common_aggregate.VisitOperator(*plan);
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
-	context.profiler.StartPhase("column_lifetime");
+	context.profiler->StartPhase("column_lifetime");
 	ColumnLifetimeAnalyzer column_lifetime(true);
 	column_lifetime.VisitOperator(*plan);
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
 	// transform ORDER BY + LIMIT to TopN
-	context.profiler.StartPhase("top_n");
+	context.profiler->StartPhase("top_n");
 	TopN topn;
 	plan = topn.Optimize(move(plan));
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
 	// apply simple expression heuristics to get an initial reordering
-	context.profiler.StartPhase("reorder_filter");
+	context.profiler->StartPhase("reorder_filter");
 	ExpressionHeuristics expression_heuristics(*this);
 	plan = expression_heuristics.Rewrite(move(plan));
-	context.profiler.EndPhase();
+	context.profiler->EndPhase();
 
 	return plan;
 }
