@@ -21,14 +21,14 @@ unique_ptr<BaseStatistics> StatisticsPropagator::PropagateExpression(BoundOperat
 	}
 	switch (expr.type) {
 	case ExpressionType::OPERATOR_IS_NULL:
-		if (!child_stats[0]->has_null) {
+		if (!child_stats[0]->CanHaveNull()) {
 			// child has no null values: x IS NULL will always be false
 			*expr_ptr = make_unique<BoundConstantExpression>(Value::BOOLEAN(false));
 			return PropagateExpression(*expr_ptr);
 		}
 		return nullptr;
 	case ExpressionType::OPERATOR_IS_NOT_NULL:
-		if (!child_stats[0]->has_null) {
+		if (!child_stats[0]->CanHaveNull()) {
 			// child has no null values: x IS NOT NULL will always be true
 			*expr_ptr = make_unique<BoundConstantExpression>(Value::BOOLEAN(true));
 			return PropagateExpression(*expr_ptr);
