@@ -4,7 +4,7 @@
 namespace duckdb {
 
 TableStarExpression::TableStarExpression(string relation_name)
-    : ParsedExpression(ExpressionType::TABLE_STAR, ExpressionClass::TABLE_STAR), relation_name(relation_name) {
+    : ParsedExpression(ExpressionType::TABLE_STAR, ExpressionClass::TABLE_STAR), relation_name(move(relation_name)) {
 }
 
 string TableStarExpression::ToString() const {
