@@ -27,8 +27,116 @@ WindowExpression::WindowExpression(ExpressionType type, string schema, const str
 	}
 }
 
+string WindowExpression::GetName() const {
+	return !alias.empty() ? alias : function_name;
+}
+
 string WindowExpression::ToString() const {
-	return "WINDOW";
+	// Start with function call
+	string result = function_name + "(";
+	result += StringUtil::Join(children, children.size(), ", ",
+	                           [](const unique_ptr<ParsedExpression> &child) { return child->ToString(); });
+	// Lead/Lag extra arguments
+	if (offset_expr.get()) {
+		result += ", ";
+		result += offset_expr->ToString();
+	}
+	if (default_expr.get()) {
+		result += ", ";
+		result += default_expr->ToString();
+	}
+	// Over clause
+	result += ") OVER(";
+	string sep;
+
+	// Partitions
+	if (!partitions.empty()) {
+		result += "PARTITION BY ";
+		result += StringUtil::Join(partitions, partitions.size(), ", ",
+		                           [](const unique_ptr<ParsedExpression> &partition) { return partition->ToString(); });
+		sep = " ";
+	}
+
+	// Orders
+	if (!orders.empty()) {
+		result += sep;
+		result += "ORDER BY ";
+		result +=
+		    StringUtil::Join(orders, orders.size(), ", ", [](const OrderByNode &order) { return order.ToString(); });
+		sep = " ";
+	}
+
+	// Rows/Range
+	string units = "ROWS";
+	string from;
+	switch (start) {
+	case WindowBoundary::CURRENT_ROW_RANGE:
+		from = "CURRENT ROW";
+		units = "RANGE";
+		break;
+	case WindowBoundary::CURRENT_ROW_ROWS:
+		from = "CURRENT ROW";
+		units = "ROWS";
+		break;
+	case WindowBoundary::UNBOUNDED_PRECEDING:
+		if (end != WindowBoundary::CURRENT_ROW_RANGE) {
+			from = "UNBOUNDED PRECEDING";
+		}
+		break;
+	case WindowBoundary::EXPR_PRECEDING:
+		from = start_expr->ToString() + " PRECEDING";
+		break;
+	case WindowBoundary::EXPR_FOLLOWING:
+		from = start_expr->ToString() + " FOLLOWING";
+		break;
+	default:
+		break;
+	}
+
+	string to;
+	switch (end) {
+	case WindowBoundary::CURRENT_ROW_RANGE:
+		if (start != WindowBoundary::UNBOUNDED_PRECEDING) {
+			to = "CURRENT ROW";
+			units = "RANGE";
+		}
+		break;
+	case WindowBoundary::CURRENT_ROW_ROWS:
+		to = "CURRENT ROW";
+		units = "ROWS";
+		break;
+	case WindowBoundary::UNBOUNDED_PRECEDING:
+		to = "UNBOUNDED PRECEDING";
+		break;
+	case WindowBoundary::EXPR_PRECEDING:
+		to = end_expr->ToString() + " PRECEDING";
+		break;
+	case WindowBoundary::EXPR_FOLLOWING:
+		to = end_expr->ToString() + " FOLLOWING";
+		break;
+	default:
+		break;
+	}
+
+	if (!from.empty() || !to.empty()) {
+		result += sep + units;
+	}
+	if (!from.empty() && !to.empty()) {
+		result += " BETWEEN ";
+		result += from;
+		result += " AND ";
+		result += to;
+	} else if (!from.empty()) {
+		result += " ";
+		result += from;
+	} else if (!to.empty()) {
+		result += " ";
+		result += to;
+	}
+
+	result += ")";
+
+	return result;
 }
 
 bool WindowExpression::Equals(const WindowExpression *a, const WindowExpression *b) {
