@@ -90,6 +90,12 @@ void ParsedExpressionIterator::EnumerateChildren(
 		for (auto &child : window_expr.children) {
 			callback(child);
 		}
+		if (window_expr.start_expr) {
+			callback(window_expr.start_expr);
+		}
+		if (window_expr.end_expr) {
+			callback(window_expr.end_expr);
+		}
 		if (window_expr.offset_expr) {
 			callback(window_expr.offset_expr);
 		}
@@ -105,6 +111,7 @@ void ParsedExpressionIterator::EnumerateChildren(
 	case ExpressionClass::STAR:
 	case ExpressionClass::TABLE_STAR:
 	case ExpressionClass::PARAMETER:
+	case ExpressionClass::POSITIONAL_REFERENCE:
 		// these node types have no children
 		break;
 	default:
