@@ -2,7 +2,8 @@
 
 namespace duckdb {
 
-CopyStatement::CopyStatement() : SQLStatement(StatementType::COPY_STATEMENT), info(make_unique<CopyInfo>()){};
+CopyStatement::CopyStatement() : SQLStatement(StatementType::COPY_STATEMENT), info(make_unique<CopyInfo>()) {
+}
 
 unique_ptr<SQLStatement> CopyStatement::Copy() const {
 	auto result = make_unique<CopyStatement>();
