@@ -2,7 +2,8 @@
 
 namespace duckdb {
 
-VacuumStatement::VacuumStatement() : SQLStatement(StatementType::VACUUM_STATEMENT){};
+VacuumStatement::VacuumStatement() : SQLStatement(StatementType::VACUUM_STATEMENT) {
+}
 
 unique_ptr<SQLStatement> VacuumStatement::Copy() const {
 	return make_unique<VacuumStatement>();
