@@ -18,14 +18,14 @@ unique_ptr<Constraint> Transformer::TransformConstraint(duckdb_libpgquery::PGLis
 		return make_unique<UniqueConstraint>(columns, is_primary_key);
 	}
 	case duckdb_libpgquery::PG_CONSTR_CHECK: {
-		auto expression = TransformExpression(constraint->raw_expr);
+		auto expression = TransformExpression(constraint->raw_expr, 0);
 		if (expression->HasSubquery()) {
 			throw ParserException("subqueries prohibited in CHECK constraints");
 		}
 		if (expression->IsAggregate()) {
 			throw ParserException("aggregates prohibited in CHECK constraints");
 		}
-		return make_unique<CheckConstraint>(TransformExpression(constraint->raw_expr));
+		return make_unique<CheckConstraint>(TransformExpression(constraint->raw_expr, 0));
 	}
 	default:
 		throw NotImplementedException("Constraint type not handled yet!");
@@ -48,7 +48,7 @@ unique_ptr<Constraint> Transformer::TransformConstraint(duckdb_libpgquery::PGLis
 	case duckdb_libpgquery::PG_CONSTR_NULL:
 		return nullptr;
 	case duckdb_libpgquery::PG_CONSTR_DEFAULT:
-		column.default_value = TransformExpression(constraint->raw_expr);
+		column.default_value = TransformExpression(constraint->raw_expr, 0);
 		return nullptr;
 	case duckdb_libpgquery::PG_CONSTR_FOREIGN:
 	default:
