@@ -40,7 +40,7 @@ unique_ptr<QueryNode> Transformer::TransformSelectNode(duckdb_libpgquery::PGSele
 			auto target = reinterpret_cast<duckdb_libpgquery::PGNode *>(stmt->distinctClause->head->data.ptr_value);
 			if (target) {
 				//  add the columns defined in the ON clause to the select list
-				if (!TransformExpressionList(stmt->distinctClause, modifier->distinct_on_targets)) {
+				if (!TransformExpressionList(stmt->distinctClause, modifier->distinct_on_targets, 0)) {
 					throw Exception("Failed to transform expression list from DISTINCT ON.");
 				}
 			}
@@ -58,18 +58,18 @@ unique_ptr<QueryNode> Transformer::TransformSelectNode(duckdb_libpgquery::PGSele
 				throw ParserException("SELECT clause without selection list");
 			}
 			// select list
-			if (!TransformExpressionList(stmt->targetList, result->select_list)) {
+			if (!TransformExpressionList(stmt->targetList, result->select_list, 0)) {
 				throw InternalException("Failed to transform expression list.");
 			}
 			result->from_table = TransformFrom(stmt->fromClause);
 		}
 
 		// where
-		result->where_clause = TransformExpression(stmt->whereClause);
+		result->where_clause = TransformExpression(stmt->whereClause, 0);
 		// group by
 		TransformGroupBy(stmt->groupClause, result->groups);
 		// having
-		result->having = TransformExpression(stmt->havingClause);
+		result->having = TransformExpression(stmt->havingClause, 0);
 		// sample
 		result->sample = TransformSampleOptions(stmt->sampleOptions);
 		break;
@@ -126,10 +126,10 @@ unique_ptr<QueryNode> Transformer::TransformSelectNode(duckdb_libpgquery::PGSele
 	if (stmt->limitCount || stmt->limitOffset) {
 		auto limit_modifier = make_unique<LimitModifier>();
 		if (stmt->limitCount) {
-			limit_modifier->limit = TransformExpression(stmt->limitCount);
+			limit_modifier->limit = TransformExpression(stmt->limitCount, 0);
 		}
 		if (stmt->limitOffset) {
-			limit_modifier->offset = TransformExpression(stmt->limitOffset);
+			limit_modifier->offset = TransformExpression(stmt->limitOffset, 0);
 		}
 		node->modifiers.push_back(move(limit_modifier));
 	}
