@@ -2,6 +2,10 @@
 
 namespace duckdb {
 
+BoundCaseExpression::BoundCaseExpression(LogicalType type)
+    : Expression(ExpressionType::CASE_EXPR, ExpressionClass::BOUND_CASE, move(type)) {
+}
+
 BoundCaseExpression::BoundCaseExpression(unique_ptr<Expression> check, unique_ptr<Expression> res_if_true,
                                          unique_ptr<Expression> res_if_false)
     : Expression(ExpressionType::CASE_EXPR, ExpressionClass::BOUND_CASE, res_if_true->return_type), check(move(check)),
@@ -13,11 +17,11 @@ string BoundCaseExpression::ToString() const {
 	       result_if_false->GetName() + ")";
 }
 
-bool BoundCaseExpression::Equals(const BaseExpression *other_) const {
-	if (!Expression::Equals(other_)) {
+bool BoundCaseExpression::Equals(const BaseExpression *other_p) const {
+	if (!Expression::Equals(other_p)) {
 		return false;
 	}
-	auto other = (BoundCaseExpression *)other_;
+	auto other = (BoundCaseExpression *)other_p;
 	if (!Expression::Equals(check.get(), other->check.get())) {
 		return false;
 	}
