@@ -7,8 +7,8 @@ namespace duckdb {
 
 AlterBinder::AlterBinder(Binder &binder, ClientContext &context, string table, vector<ColumnDefinition> &columns,
                          vector<column_t> &bound_columns, LogicalType target_type)
-    : ExpressionBinder(binder, context), table(table), columns(columns), bound_columns(bound_columns) {
-	this->target_type = target_type;
+    : ExpressionBinder(binder, context), table(move(table)), columns(columns), bound_columns(bound_columns) {
+	this->target_type = move(target_type);
 }
 
 BindResult AlterBinder::BindExpression(unique_ptr<ParsedExpression> *expr_ptr, idx_t depth, bool root_expression) {
