@@ -12,15 +12,15 @@ static bool UseVersion(Transaction &transaction, transaction_t id) {
 	return UseVersion(transaction.start_time, transaction.transaction_id, id);
 }
 
-unique_ptr<ChunkInfo> ChunkInfo::Deserialize(MorselInfo &morsel, Deserializer &source) {
+unique_ptr<ChunkInfo> ChunkInfo::Deserialize(Deserializer &source) {
 	auto type = source.Read<ChunkInfoType>();
 	switch (type) {
 	case ChunkInfoType::EMPTY_INFO:
 		return nullptr;
 	case ChunkInfoType::CONSTANT_INFO:
-		return ChunkConstantInfo::Deserialize(morsel, source);
+		return ChunkConstantInfo::Deserialize(source);
 	case ChunkInfoType::VECTOR_INFO:
-		return ChunkVectorInfo::Deserialize(morsel, source);
+		return ChunkVectorInfo::Deserialize(source);
 	default:
 		throw SerializationException("Could not deserialize Chunk Info Type: unrecognized type");
 	}
@@ -29,8 +29,8 @@ unique_ptr<ChunkInfo> ChunkInfo::Deserialize(MorselInfo &morsel, Deserializer &s
 //===--------------------------------------------------------------------===//
 // Constant info
 //===--------------------------------------------------------------------===//
-ChunkConstantInfo::ChunkConstantInfo(idx_t start, MorselInfo &morsel)
-    : ChunkInfo(start, morsel, ChunkInfoType::CONSTANT_INFO), insert_id(0), delete_id(NOT_DELETED_ID) {
+ChunkConstantInfo::ChunkConstantInfo(idx_t start)
+    : ChunkInfo(start, ChunkInfoType::CONSTANT_INFO), insert_id(0), delete_id(NOT_DELETED_ID) {
 }
 
 idx_t ChunkConstantInfo::GetSelVector(Transaction &transaction, SelectionVector &sel_vector, idx_t max_count) {
@@ -60,10 +60,10 @@ void ChunkConstantInfo::Serialize(Serializer &serializer) {
 	serializer.Write<idx_t>(start);
 }
 
-unique_ptr<ChunkInfo> ChunkConstantInfo::Deserialize(MorselInfo &morsel, Deserializer &source) {
+unique_ptr<ChunkInfo> ChunkConstantInfo::Deserialize(Deserializer &source) {
 	auto start = source.Read<idx_t>();
 
-	auto info = make_unique<ChunkConstantInfo>(start, morsel);
+	auto info = make_unique<ChunkConstantInfo>(start);
 	info->insert_id = 0;
 	info->delete_id = 0;
 	return move(info);
@@ -72,8 +72,8 @@ unique_ptr<ChunkInfo> ChunkConstantInfo::Deserialize(MorselInfo &morsel, Deseria
 //===--------------------------------------------------------------------===//
 // Vector info
 //===--------------------------------------------------------------------===//
-ChunkVectorInfo::ChunkVectorInfo(idx_t start, MorselInfo &morsel)
-    : ChunkInfo(start, morsel, ChunkInfoType::VECTOR_INFO), insert_id(0), same_inserted_id(true), any_deleted(false) {
+ChunkVectorInfo::ChunkVectorInfo(idx_t start)
+    : ChunkInfo(start, ChunkInfoType::VECTOR_INFO), insert_id(0), same_inserted_id(true), any_deleted(false) {
 	for (idx_t i = 0; i < STANDARD_VECTOR_SIZE; i++) {
 		inserted[i] = 0;
 		deleted[i] = NOT_DELETED_ID;
@@ -127,11 +127,15 @@ bool ChunkVectorInfo::Fetch(Transaction &transaction, row_t row) {
 	return UseVersion(transaction, inserted[row]) && !UseVersion(transaction, deleted[row]);
 }
 
-void ChunkVectorInfo::Delete(Transaction &transaction, row_t rows[], idx_t count) {
+idx_t ChunkVectorInfo::Delete(Transaction &transaction, row_t rows[], idx_t count) {
 	any_deleted = true;
 
 	// first check the chunk for conflicts
+	idx_t deleted_tuples = 0;
 	for (idx_t i = 0; i < count; i++) {
+		if (deleted[rows[i]] == transaction.transaction_id) {
+			continue;
+		}
 		if (deleted[rows[i]] != NOT_DELETED_ID) {
 			// tuple was already deleted by another transaction
 			throw TransactionException("Conflict on tuple deletion!");
@@ -139,11 +143,13 @@ void ChunkVectorInfo::Delete(Transaction &transaction, row_t rows[], idx_t count
 		if (inserted[rows[i]] >= TRANSACTION_ID_START) {
 			throw TransactionException("Deleting non-committed tuples is not supported (for now...)");
 		}
+		deleted_tuples++;
 	}
 	// after verifying that there are no conflicts we mark the tuples as deleted
 	for (idx_t i = 0; i < count; i++) {
 		deleted[rows[i]] = transaction.transaction_id;
 	}
+	return deleted_tuples;
 }
 
 void ChunkVectorInfo::CommitDelete(transaction_t commit_id, row_t rows[], idx_t count) {
@@ -202,10 +208,10 @@ void ChunkVectorInfo::Serialize(Serializer &serializer) {
 	serializer.WriteData((data_ptr_t)deleted_tuples, sizeof(bool) * STANDARD_VECTOR_SIZE);
 }
 
-unique_ptr<ChunkInfo> ChunkVectorInfo::Deserialize(MorselInfo &morsel, Deserializer &source) {
+unique_ptr<ChunkInfo> ChunkVectorInfo::Deserialize(Deserializer &source) {
 	auto start = source.Read<idx_t>();
 
-	auto result = make_unique<ChunkVectorInfo>(start, morsel);
+	auto result = make_unique<ChunkVectorInfo>(start);
 	result->any_deleted = true;
 	bool deleted_tuples[STANDARD_VECTOR_SIZE];
 	source.ReadData((data_ptr_t)deleted_tuples, sizeof(bool) * STANDARD_VECTOR_SIZE);
